package de.pschuett.dialogs;

import javax.swing.filechooser.FileFilter;
import java.util.*;
import java.io.*;
import com.ice.util.FileUtilities;

/**
 *  Default file filter
 *
 * @author    ps
 */

public class DefaultFileFilter extends FileFilter
{
    Set mFilterStrings;
    String mDescription;
    boolean mCaseSensitive = true;

    public final static char FILTERSTRING_DELIMITER='|';

    /**
     *  Constructor for the DefaultFileFilter object
     *
     * @param  exts            array of pattern strings
     * @param  aDescription    description of filter
     * @param  aCaseSensitive  true if pattern is case sensitive
     */
    public DefaultFileFilter(String[] exts, String aDescription, boolean aCaseSensitive)
    {
        mFilterStrings = new HashSet();
        for (int ii = 0; ii < exts.length; ii++)
        {
            mFilterStrings.add(exts[ii]);
        }
        mDescription = aDescription;
        mCaseSensitive = aCaseSensitive;
    }


    /**
     *  Constructor for the DefaultFileFilter object
     *
     * @param  exts            pattern string
     * @param  aDescription    description of filter
     * @param  aCaseSensitive  true if pattern is case sensitive
     */
    public DefaultFileFilter(String exts, String aDescription, boolean aCaseSensitive)
    {
        mFilterStrings = new HashSet();
        StringTokenizer tok = new StringTokenizer(exts, ";");
        while (tok.hasMoreTokens())
        {
            mFilterStrings.add(tok.nextToken());
        }
        mDescription = aDescription;
        mCaseSensitive = aCaseSensitive;
    }

    /**
     *  Parses a string with file filter description in windows format <br>
     *  Example <br>
     *  Text-Files (*.TXT)|*.TXT|Java-Source (*.java)|*.java|Rest (*.*)|*.*|
     *
     * @param  aValue  filter description string
     * @return         array of file filters
     */
    public static FileFilter[] parseFileFilterString(String aValue)
    {
        Vector filters = new Vector();
        StringTokenizer tok = new StringTokenizer(aValue, "" + FILTERSTRING_DELIMITER);
        while (tok.hasMoreTokens())
        {
            String desc = tok.nextToken();
            String fil = tok.nextToken();
            DefaultFileFilter filter = new DefaultFileFilter(fil, desc, true);
            filters.add(filter);
        }
        FileFilter[] filterArray = new FileFilter[filters.size()];
        return (FileFilter[])filters.toArray(filterArray);
    }


    /**
     *  Gets the description of the filter
     *
     * @return    filter description
     */
    public String getDescription()
    {
        return mDescription;
    }


    /**
     *  Returns true if a file matches a pattern of this filter<br>
     *  The matching check is still not supported by standard java
     *  so class found in internet is used ({@com.ice.util.FileUtilities#matchPattern}).
     *
     * @param  f  File which must be checked
     * @return    True if the file matches this filter
     */
    public boolean accept(File f)
    {
        if (f.isDirectory())
        {
            return true;
        }
        Iterator iter = mFilterStrings.iterator();
        String mask;
        while (iter.hasNext())
        {
            mask = (String)iter.next();
            if (FileUtilities.matchPattern(f.getName(), mask))
            {
                return true;
            }
        }
        return false;
    }

}
