package de.pschuett.dialogs;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

/**
 *  Dialog for setting the filter options
 *
 * @author     ps
 * @version    1.0
 */

public class FilterDialog extends JDialog
{

    private int pressedRow = -1;
    JPanel panel1 = new JPanel();
    GridBagLayout gridBagLayout1 = new GridBagLayout();
    JPanel jButtonPanel = new JPanel();
    JScrollPane jScrollPane = new JScrollPane();
    JButton jCancelButton = new JButton();
    JButton jOkButton = new JButton();
    JTable jFilterTable = new JTable();

    boolean ok = false;


    /**
     *  Default constructor for this dialog
     *
     * @param  frame  parent window
     */
    public FilterDialog(Frame frame)
    {
        super(frame);
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try
        {
            jbInit();
            pack();
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }


    /**
     *  Initilization of the gui elements
     */
    void jbInit() throws Exception
    {
        this.setTitle("Dateifilter einstellen");
        panel1.setLayout(gridBagLayout1);
        jCancelButton.setText("Abbrechen");
        jCancelButton.addActionListener(
            new java.awt.event.ActionListener()
            {
                /**
                 *  Event when dialog is canceled
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jCancelButton_actionPerformed(e);
                }
            });
        jOkButton.setPreferredSize(new Dimension(93, 27));
        jOkButton.setText("OK");
        jOkButton.addActionListener(
            new java.awt.event.ActionListener()
            {
                /**
                 *  Event when OK button is pressed
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jOkButton_actionPerformed(e);
                }
            });
        jFilterTable.setToolTipText("Mit Einfg neue Zeile hinzufgen");
        jFilterTable.addKeyListener(new java.awt.event.KeyAdapter()
        {
            public void keyReleased(KeyEvent e)
            {
                jFilterTable_keyReleased(e);
            }
        });
        getContentPane().add(panel1);
        panel1.add(jButtonPanel, new GridBagConstraints(0, 1, 1, 1, 100.0, 0.0
                , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        jButtonPanel.add(jOkButton, null);
        jButtonPanel.add(jCancelButton, null);
        panel1.add(jScrollPane, new GridBagConstraints(0, 0, 1, 1, 100.0, 100.0
                , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        jFilterTable.addMouseMotionListener(
            new java.awt.event.MouseMotionAdapter()
            {
                /**
                 *  Mouse handler for suppressing selection during drag and drop
                 */
                public void mouseDragged(MouseEvent e)
                {
                    jFilterTable_mouseDragged(e);
                }
            });
        jFilterTable.addMouseListener(
            new java.awt.event.MouseAdapter()
            {
                /**
                 *  Mouse handler to start drag and drop
                 */
                public void mousePressed(MouseEvent e)
                {
                    jFilterTable_mousePressed(e);
                }


                /**
                 *  Mouse handler for finishing drag and drop
                 */
                public void mouseReleased(MouseEvent e)
                {
                    jFilterTable_mouseReleased(e);
                }
            });
        jScrollPane.getViewport().add(jFilterTable, null);
        ((DefaultTableModel)jFilterTable.getModel()).setColumnIdentifiers(new Object[]{" ", "Filtername", "Filter"});
        TableColumn column = jFilterTable.getColumn(" ");
        column.setMaxWidth(jFilterTable.getRowHeight());
        column.setMinWidth(column.getMaxWidth());
        column.setPreferredWidth(column.getMaxWidth());
        column.setResizable(false);
        // only necessary for JDK 1.2.*
        try
        {
            jFilterTable.getTableHeader().setReorderingAllowed(false);
        }
        catch (Exception ex)
        {
        }
    }


    /**
     *  cancel dialog
     */
    void cancel()
    {
        dispose();
    }


    /**
     *  Cancel event method
     */
    void jCancelButton_actionPerformed(ActionEvent e)
    {
        cancel();
    }


    /**
     *  OK event method
     */
    void jOkButton_actionPerformed(ActionEvent e)
    {
        if (jFilterTable.getCellEditor() != null)
        {
            jFilterTable.getCellEditor().stopCellEditing();
        }
        ok = true;
        dispose();
    }


    /**
     *  Beginning of drag and drop
     */
    void jFilterTable_mousePressed(MouseEvent e)
    {
        if (jFilterTable.columnAtPoint(new Point(e.getX(), e.getY())) != 0)
        {
            return;
        }
        //jFilterTable.clearSelection();
        pressedRow = jFilterTable.rowAtPoint(new Point(e.getX(), e.getY()));
        setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
    }


    /**
     *  Finishing of drag and drop
     */
    void jFilterTable_mouseReleased(MouseEvent e)
    {
        setCursor(Cursor.getDefaultCursor());
        if (pressedRow == -1)
        {
            return;
        }
        int row = jFilterTable.rowAtPoint(new Point(e.getX(), e.getY()));
        int col = jFilterTable.columnAtPoint(new Point(e.getX(), e.getY()));
        if ((row != pressedRow) && (col >= 0))
        {
            ((DefaultTableModel)jFilterTable.getModel()).moveRow(pressedRow, pressedRow, row);
        }
        pressedRow = -1;
    }


    /**
     *  Suppressed selection during drag and drop
     */
    void jFilterTable_mouseDragged(MouseEvent e)
    {
        if (pressedRow != -1)
        {
            jFilterTable.clearSelection();
        }
    }


    /**
     *  It has to be possible to finish this dialog.
     */
    protected void processWindowEvent(WindowEvent e)
    {
        if (e.getID() == WindowEvent.WINDOW_CLOSING)
        {
            cancel();
        }
        super.processWindowEvent(e);
    }


    /**
     *  Returns the filter string out of the table
     *
     * @return    the filter string
     */
    public String getFilterString()
    {
        StringBuffer result = new StringBuffer("");
        TableModel model = jFilterTable.getModel();
        for (int row = 0; row < model.getRowCount(); row++)
        {
            String filterName = (String)model.getValueAt(row, 1);
            String filter = (String)model.getValueAt(row, 2);
            if (((filterName != null) && (filterName.length() > 0))
                     || ((filter != null) && (filter.length() > 0)))
            {
                result.append(filterName);
                result.append(DefaultFileFilter.FILTERSTRING_DELIMITER);
                result.append(filter);
                result.append(DefaultFileFilter.FILTERSTRING_DELIMITER);
            }
        }
        return result.toString();
    }


    /**
     *  method for showing the dialog<br>
     *  The table is initialized by the given filter string.
     *
     * @param  aFilterString  filter string
     * @return                True if OK is pressed
     */
    public boolean showDialog(String aFilterString)
    {
        StringTokenizer tok = new StringTokenizer(aFilterString, "" + DefaultFileFilter.FILTERSTRING_DELIMITER);
        while (tok.hasMoreTokens())
        {
            String filterName = tok.nextToken();
            String filterStr = tok.nextToken();
            ((DefaultTableModel)jFilterTable.getModel()).addRow(new Object[]{" ", filterName, filterStr});
        }
        show();
        return ok;
    }

    void jFilterTable_keyReleased(KeyEvent e)
    {
        if ((e.getKeyCode() == KeyEvent.VK_INSERT) && (e.getModifiers() == 0))
        {
            int row = jFilterTable.getSelectedRow();
            if (row < 0)
                ((DefaultTableModel)jFilterTable.getModel()).addRow(new Object[]{" ", "", ""});
            else
                ((DefaultTableModel)jFilterTable.getModel()).insertRow(row, new Object[]{" ", "", ""});
        }
    }

}
