package de.pschuett.dialogs;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;
import java.awt.event.*;
import javax.swing.event.*;

/**
 *  Dialog for choosing a font
 *
 * @author    ps
 */
public class FontChooser extends JDialog
{
    private Font font = null;
    private boolean ok = false;
    private ListCellRenderer defaultListRenderer;

    JPanel contentPanel = new JPanel();
    BorderLayout borderLayout1 = new BorderLayout();
    JPanel jButtonPanel = new JPanel();
    JButton jCancelButton = new JButton();
    JButton jOkButton = new JButton();
    JButton jHelpButton = new JButton();
    JPanel jAttributesPanel = new JPanel();
    JTextArea jExampleTextArea = new JTextArea();
    JTextField jSizeTextField = new JTextField(new IntTextDocument(), null, 0);
    JLabel jLabel1 = new JLabel();
    Border border1;
    BorderLayout borderLayout2 = new BorderLayout();
    String[] fontNames = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
    JPanel jPanel1 = new JPanel();
    JScrollPane jFontScrollPane = new JScrollPane();
    JList jFontList = new JList(fontNames);
    BorderLayout borderLayout3 = new BorderLayout();
    JCheckBox jNamesBox = new JCheckBox();
    GridBagLayout gridBagLayout1 = new GridBagLayout();
    JCheckBox jBoldBox = new JCheckBox();
    JCheckBox jItalicBox = new JCheckBox();


    /**
     *  Default constructor
     *
     * @param  parent  parent window
     */
    public FontChooser(Frame parent)
    {
        super(parent);
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try
        {
            jbInit();
            pack();
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }


    /**
     *  Creates a new font by the selected values of this dialog
     *
     * @return    Returns new font
     */
    private Font createNewFont()
    {
        int fontStyle;
        fontStyle = Font.PLAIN;
        if (jItalicBox.isSelected())
        {
            fontStyle += Font.ITALIC;
        }
        if (jBoldBox.isSelected())
        {
            fontStyle += Font.BOLD;
        }
        return new Font((String)jFontList.getSelectedValue(), fontStyle, new Integer(jSizeTextField.getText()).intValue());
    }


    /**
     *  updates example text
     */
    private void updateExampleText()
    {
        jExampleTextArea.setFont(createNewFont());
    }


    /**
     *  Initialization of the gui elements
     */
    void jbInit() throws Exception
    {
        border1 = new EtchedBorder(EtchedBorder.RAISED, Color.white, new Color(178, 178, 178));
        contentPanel.setLayout(borderLayout1);
        jCancelButton.setText("Abbruch");
        jCancelButton.addActionListener(
            new java.awt.event.ActionListener()
            {
                /**
                 *  {@link FontChooser#jCancelButton_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jCancelButton_actionPerformed(e);
                }
            });
        jOkButton.setText("OK");
        jOkButton.addActionListener(
            new java.awt.event.ActionListener()
            {
                /**
                 *  {@link FontChooser#jOkButton_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jOkButton_actionPerformed(e);
                }
            });
        jHelpButton.setText("Hilfe");
        jSizeTextField.setMinimumSize(new Dimension(18, 21));
        jSizeTextField.setText("12");
        jSizeTextField.addKeyListener(
            new java.awt.event.KeyAdapter()
            {
                /**
                 *  {@link FontChooser#jSizeTextField_keyReleased}
                 */
                public void keyReleased(KeyEvent e)
                {
                    jSizeTextField_keyReleased(e);
                }
            });
        jLabel1.setText("Schriftgre");
        jExampleTextArea.setPreferredSize(new Dimension(90, 70));
        jExampleTextArea.setWrapStyleWord(true);
        jExampleTextArea.setLineWrap(true);
        jExampleTextArea.setBorder(border1);
        jExampleTextArea.setMinimumSize(new Dimension(90, 15));
        jExampleTextArea.setText("Hier knnen Sie den gewhlten Zeichensatz ausprobieren.");
        jExampleTextArea.setMaximumSize(new Dimension(90, 70));
        this.getContentPane().setBackground(Color.white);
        this.setModal(true);
        this.setTitle("Schriftart auswhlen");
        this.getContentPane().setLayout(borderLayout2);
        jPanel1.setLayout(borderLayout3);
        jNamesBox.setText("Nur Namen anzeigen");
        jNamesBox.addActionListener(
            new java.awt.event.ActionListener()
            {
                /**
                 *  {@link FontChooser#jNamesBox_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jNamesBox_actionPerformed(e);
                }
            });
        jFontList.addListSelectionListener(
            new javax.swing.event.ListSelectionListener()
            {
                /**
                 *  {@link FontChooser#jFontList_valueChanged}
                 */
                public void valueChanged(ListSelectionEvent e)
                {
                    jFontList_valueChanged(e);
                }
            });
        jAttributesPanel.setLayout(gridBagLayout1);
        jAttributesPanel.setMaximumSize(new Dimension(110, 42));
        jAttributesPanel.setMinimumSize(new Dimension(110, 42));
        jAttributesPanel.setPreferredSize(new Dimension(110, 42));
        jBoldBox.setText("Fett");
        jBoldBox.addActionListener(
            new java.awt.event.ActionListener()
            {
                /**
                 *  {@link FontChooser#jBoldBox_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jBoldBox_actionPerformed(e);
                }
            });
        jItalicBox.setText("Kursiv");
        jItalicBox.addActionListener(
            new java.awt.event.ActionListener()
            {
                /**
                 *  {@link FontChooser#jItalicBox_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jItalicBox_actionPerformed(e);
                }
            });
        getContentPane().add(contentPanel, BorderLayout.CENTER);
        contentPanel.add(jAttributesPanel, BorderLayout.EAST);
        jAttributesPanel.add(jExampleTextArea, new GridBagConstraints(0, 3, 3, 1, 100.0, 100.0
                , GridBagConstraints.SOUTH, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 19, 30));
        jAttributesPanel.add(jLabel1, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        jAttributesPanel.add(jBoldBox, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        jAttributesPanel.add(jSizeTextField, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        jAttributesPanel.add(jItalicBox, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        contentPanel.add(jPanel1, BorderLayout.CENTER);
        jPanel1.add(jFontScrollPane, BorderLayout.CENTER);
        jFontScrollPane.getViewport().add(jFontList, null);
        jPanel1.add(jNamesBox, BorderLayout.SOUTH);
        contentPanel.add(jButtonPanel, BorderLayout.SOUTH);
        jButtonPanel.add(jOkButton, null);
        jButtonPanel.add(jCancelButton, null);
        jButtonPanel.add(jHelpButton, null);
        defaultListRenderer = jFontList.getCellRenderer();
        jNamesBox.setSelected(true);
        //jFontList.setCellRenderer(new FontCellRenderer());
    }


    /**
     *  Cancel dialog
     */
    void cancel()
    {
        dispose();
    }


    /**
     *  Dialog was canceled
     */
    void jCancelButton_actionPerformed(ActionEvent e)
    {
        cancel();
    }


    /**
     *  Ok button was pressed.
     */
    void jOkButton_actionPerformed(ActionEvent e)
    {
        ok = true;
        font = createNewFont();
        dispose();
    }


    /**
     *  Event if font names should be shown as plain text or in their own font
     */
    void jNamesBox_actionPerformed(ActionEvent e)
    {
        if (jNamesBox.isSelected())
        {
            jFontList.setCellRenderer(defaultListRenderer);
        }
        else
        {
            jFontList.setCellRenderer(new FontCellRenderer());
        }
        Cursor cursor = getCursor();
        try
        {
            setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            jFontList.update(jFontList.getGraphics());
        }
        finally
        {
            setCursor(cursor);
        }
    }


    /**
     * Another font was choose.<br>
     * The example text is updated.
     */
    void jFontList_valueChanged(ListSelectionEvent e)
    {
        updateExampleText();
    }


    /**
     * The bold box was clicked.<br>
     * The example text is updated.
     */
    void jBoldBox_actionPerformed(ActionEvent e)
    {
        updateExampleText();
    }


    /**
     * The italic box was clicked.<br>
     * The example text is updated.
     */
    void jItalicBox_actionPerformed(ActionEvent e)
    {
        updateExampleText();
    }


    /**
     * The font size was changed.<br>
     * The example text is updated.
     */
    void jSizeTextField_keyReleased(KeyEvent e)
    {
        updateExampleText();
    }


    /**
     *  Make finishing of this dialog possible.
     */
    protected void processWindowEvent(WindowEvent e)
    {
        if (e.getID() == WindowEvent.WINDOW_CLOSING)
        {
            cancel();
        }
        super.processWindowEvent(e);
    }


    /**
     *  Returns the new font with the new configuration
     *
     * @return    new font
     */
    public Font getFont()
    {
        return font;
    }


    /**
     *  Default method to show this dialog
     *
     * @param  aFont  old font
     * @return        True if OK is pressed.
     */
    public boolean showDialog(Font aFont)
    {
        font = aFont;
        jFontList.setSelectedValue(font.getName(), true);
        show();
        return ok;
        // if ok then aFont is set to the selected font.
    }


    /**
     *  New document to force an integer in jSizeTextField
     *
     * @author    pschuett
     */
    private static class IntTextDocument extends PlainDocument
    {
        /**
         *  See {@link javax.swing.text.PlainDocument#insertString}
         */
        public void insertString(int offs, String str, AttributeSet a) throws BadLocationException
        {
            if (str == null)
            {
                return;
            }
            String oldString = getText(0, getLength());
            String newString = oldString.substring(0, offs) + str + oldString.substring(offs);
            try
            {
                int nn = Integer.parseInt(newString + "0");
                if (nn <= 0)
                {
                    return;
                }
                super.insertString(offs, str, a);
            }
            catch (NumberFormatException e)
            {
            }
        }
    }


    /**
     *  New renderer to show every font name in its own font
     *
     * @author    pschuett
     */
    private class FontCellRenderer implements ListCellRenderer
    {
        /**
         *  See {@link javax.swing.ListCellRenderer#getListCellRendererComponent}
         */
        public Component getListCellRendererComponent
                (final JList list, final Object value,
                final int index, final boolean isSelected,
                final boolean cellHasFocus)
        {
            return
                new JPanel()
                {

                    /**
                     *  Gets the preferred size attribute of a list item
                     *
                     * @see     javax.swing.JPanel
                     * @return    The preferred size value
                     */
                    public Dimension getPreferredSize()
                    {
                        Font font = new Font((String)value, Font.PLAIN, list.getFont().getSize());
                        String text = font.getFamily();
                        Graphics g = getGraphics();
                        FontMetrics fm = g.getFontMetrics(font);
                        return new Dimension(fm.stringWidth(text),
                                fm.getHeight());
                    }


                    /**
                     *  Paint method for a list item
                     */
                    public void paintComponent(Graphics g)
                    {
                        Font font = new Font((String)value, Font.PLAIN, list.getFont().getSize());
                        String text = font.getFamily();
                        FontMetrics fm = g.getFontMetrics(font);
                        g.setColor(isSelected
                                 ? list.getSelectionBackground()
                                 : list.getBackground());
                        g.fillRect(0, 0, getWidth(), getHeight());
                        g.setColor(isSelected
                                 ? list.getSelectionForeground()
                                 : list.getForeground());
                        g.setFont(font);
                        g.drawString(text, 0, fm.getAscent());
                    }
                };
        }
    }
}
