package de.pschuett.dialogs;

import java.awt.*;
import javax.swing.*;

import java.io.*;
import java.net.URL;
import javax.swing.event.*;
import javax.swing.text.html.HTMLDocument;
import java.awt.event.*;

import java.util.*;

/**
 *  Frame to show a HTML help file<br>
 *  Launching an external browser cannot be implemented platform independed so
 *  because of the small help file a mini browser is used.<br>
 *  Only the method {@link #showHTMLHelp} has to be called to use this class.<br>
 *  This class is a singleton.
 *
 * @author     ps
 * @version    1.0
 */

public class HelpFrame extends JFrame
{

    /**
     * @link
     * @shapeType       PatternLink
     * @pattern         Singleton
     * @supplierRole    Singleton factory
     */
    /*
     * # private HelpFrame _helpFrame;
     */
    private static HelpFrame instance = null;
    JScrollPane jScrollPane = new JScrollPane();
    GridBagLayout gridBagLayout1 = new GridBagLayout();

    ImageIcon iconHome    = new ImageIcon(this.getClass().getResource("/de/pschuett/images/BrowserHome.gif"));
    ImageIcon iconBack    = new ImageIcon(this.getClass().getResource("/de/pschuett/images/BrowserBack.gif"));
    ImageIcon iconForward = new ImageIcon(this.getClass().getResource("/de/pschuett/images/BrowserForward.gif"));
    JButton mButtonHome = new JButton();
    JButton mButtonBack = new JButton();
    JButton mButtonForward = new JButton();
    JPanel mButtonPanel = new JPanel();
    JEditorPane jHTMLEditorPane = new JEditorPane();
    java.util.List  mUrlList;
    int   mCurrentIndex = -1;
    JFrame mParent  = null;

    /**
     *  Default constructor
     */
    protected HelpFrame()
    {
        try
        {
            jbInit();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }


    /**
     *  Gets the instance of this class
     *
     * @return    The instance value
     */
    private static HelpFrame getInstance()
    {
        if (instance == null)
        {
            instance = new HelpFrame();
        }
        return instance;
    }

    private void center()
    {
        if (mParent == null) return;
        Dimension dlgSize = this.getSize();
        Dimension frmSize = mParent.getSize();
        Point loc         = mParent.getLocation();
        setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    }


    /**
     *  This static method is called to show a help file.
     *
     * @param  helpURL  URL of the help file
     * @param  title    title of this frame
     * @param  icon     icon of this frame
     * @param  parent   parent main windows
     */
    public static void showHTMLHelp(String helpURL, String title, ImageIcon icon, JFrame parent)
    {
        HelpFrame helpFrame = getInstance();
        try
        {
            helpFrame.mParent = parent;
            helpFrame.mUrlList = new ArrayList();
            helpFrame.mUrlList.add(new URL(helpURL));
            helpFrame.mCurrentIndex = helpFrame.mUrlList.size() - 1;
            helpFrame.setEnabledButtons();
            helpFrame.jHTMLEditorPane.setPage(helpURL);
            if (title == null)
            {
                title = "Hilfe";
            }
            if (icon != null)
            {
                helpFrame.setIconImage(icon.getImage());
            }
            helpFrame.setTitle(title);
            // do not know how to use the title tag
            helpFrame.center();
            helpFrame.show();
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }


    /**
     *  The main program for the HelpFrame class<br>
     *  This class can be used for his own to show HTML files.
     *
     * @param  args  The command line arguments
     */
    public static void main(String[] args)
    {
        showHTMLHelp(args[0], null, null, null);
    }


    /**
     *  Initialization of the gui elements
     */
    private void jbInit() throws Exception
    {
        this.setSize(300, 400);
        this.getContentPane().setLayout(gridBagLayout1);
        mButtonHome.setIcon(iconHome);
        mButtonHome.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                mButtonHome_actionPerformed(e);
            }
        });
        mButtonBack.setIcon(iconBack);
        mButtonBack.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                mButtonBack_actionPerformed(e);
            }
        });
        mButtonForward.setIcon(iconForward);
        mButtonForward.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                mButtonForward_actionPerformed(e);
            }
        });
        mButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        jHTMLEditorPane.addHyperlinkListener(new javax.swing.event.HyperlinkListener()
            {
                public void hyperlinkUpdate(HyperlinkEvent e)
                {
                    jHTMLEditorPane_hyperlinkUpdate(e);
                }
            });
        jHTMLEditorPane.setEditable(false);
        this.getContentPane().add(jScrollPane, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        jScrollPane.getViewport().add(jHTMLEditorPane, null);
        this.getContentPane().add(mButtonPanel, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        mButtonPanel.add(mButtonHome, null);
        mButtonPanel.add(mButtonBack, null);
        mButtonPanel.add(mButtonForward, null);
    }


    /**
     *  Event handler when a link is clicked on.
     */
    void jHTMLEditorPane_hyperlinkUpdate(HyperlinkEvent e)
    {
        try
        {
            if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
            {
                for (int ii=mUrlList.size()-1; ii>mCurrentIndex; ii--)
                    mUrlList.remove(ii);
                mUrlList.add(e.getURL());
                mCurrentIndex = mUrlList.size() - 1;
                jHTMLEditorPane.setPage(e.getURL());
            }
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }
        setEnabledButtons();
    }

    void mButtonBack_actionPerformed(ActionEvent e)
    {
        doBack();
        setEnabledButtons();
    }

    /**
     * Invoked by the BackButton event
     */
    private void doBack()
    {
        int tmpBack = mCurrentIndex - 1;

        if  (mUrlList != null && tmpBack >= 0)
        {
            try
            {
                jHTMLEditorPane.setPage((URL) mUrlList.get(tmpBack));
                mCurrentIndex = tmpBack;
            }
            catch(java.io.IOException e)
            {
                e.printStackTrace();
            }
        }
    }

    /**
     * set the buttons enabled or disabled
     * depending on the model mUrlList
     */
    private void setEnabledButtons()
    {
        // HomeButton - Allways true
        mButtonHome.setEnabled(true);

        // BackButton
        if  (mUrlList != null && mCurrentIndex > 0)
        {
            mButtonBack.setEnabled(true);
        }
        else
        {
            mButtonBack.setEnabled(false);
        }

        // ForwardButton
        if  (mUrlList != null && mCurrentIndex > -1 && mCurrentIndex < mUrlList.size() - 1)
        {
            mButtonForward.setEnabled(true);
        }
        else
        {
            mButtonForward.setEnabled(false);
        }
    }

    void mButtonHome_actionPerformed(ActionEvent e)
    {
        doHome();
        setEnabledButtons();
    }

    private void doHome()
    {
        if  (mUrlList != null && mUrlList.size() > 0)
        {
            try
            {
                jHTMLEditorPane.setPage((URL) mUrlList.get(0));
                mCurrentIndex = 0;
            }
            catch(java.io.IOException e)
            {
                e.printStackTrace();
            }
        }
    }

    void mButtonForward_actionPerformed(ActionEvent e)
    {
        doForward();
        setEnabledButtons();
    }

    private void doForward()
    {
        int tmpForward = mCurrentIndex + 1;

        if  (mUrlList != null && tmpForward < mUrlList.size())
        {
            try
            {
                jHTMLEditorPane.setPage((URL) mUrlList.get(tmpForward));
                mCurrentIndex = tmpForward;
            }
            catch(java.io.IOException e)
            {
                e.printStackTrace();
            }
        }
    }
}
