package de.pschuett.editor;

import java.awt.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.print.*;
import javax.swing.*;

import java.io.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.undo.UndoManager;
import javax.swing.filechooser.FileFilter;

import java.util.Date;
import java.security.AccessControlException;
import java.text.*;
import java.util.*;

import de.pschuett.util.*;
import de.pschuett.find.*;
import de.pschuett.dialogs.*;
import de.pschuett.images.*;

/**
 *  Peters Multi-Editor <br>
 *  This is the main formular of this program.
 *
 * @author     ps
 * @version    1.0
 */
public class EditorFrame extends JFrame
{

    private final String UNNAMED = "<unbenannt>";
    private boolean currentModified = false;
    private boolean overwriteMode = false;
    private Font defaultFont = null;
    private boolean allWordWrap = false;
    private String iniFileName = "PeteEdit.ini";
    private PageFormat defaultPageFormat = null;
    private FindOptions findOptions = new EditorFindOptions();
    private String filterString = null;
    private FileFilter[] filters = null;
    private FileFilter defaultFilter = null;

    JPanel contentPane;
    JMenuBar jMenuBar1 = new JMenuBar();
    JMenu jMenuFile = new JMenu();
    JMenuItem jMenuFileExit = new JMenuItem();
    JMenu jMenuHelp = new JMenu();
    JMenuItem jMenuHelpAbout = new JMenuItem();
    JToolBar jToolBar = new JToolBar();
    JButton jOpenButton = new JButton();
    JButton jSaveButton = new JButton();
    JButton jHelpButton = new JButton();
    JButton jSaveAsButton = new JButton();
    ImageIcon newIcon;
    ImageIcon openIcon;
    ImageIcon saveIcon;
    ImageIcon saveasIcon;
    ImageIcon closeIcon;
    ImageIcon printIcon;
    ImageIcon copyIcon;
    ImageIcon cutIcon;
    ImageIcon pasteIcon;
    ImageIcon findIcon;
    ImageIcon findnextIcon;
    ImageIcon gotoIcon;
    ImageIcon undoIcon;
    ImageIcon fontIcon;
    ImageIcon helpIcon;
    ImageIcon logoIcon;
    JPanel statusPane = new JPanel();
    JLabel statusBar = new JLabel();
    BorderLayout borderLayout1 = new BorderLayout();
    JMenuItem jMenuFileSave = new JMenuItem();
    JMenuItem jMenuFileOpen = new JMenuItem();
    JMenuItem jMenuFileSaveAs = new JMenuItem();

    // instance variables
    GridBagLayout gridBagLayout = new GridBagLayout();
    JLabel jXPosLabel = new JLabel();
    JLabel jYPosLabel = new JLabel();
    JLabel jSizeLabel = new JLabel();
    JLabel jPosLabel = new JLabel();
    JMenuItem jMenuFileNew = new JMenuItem();
    JMenuItem jMenuClose = new JMenuItem();
    JMenuItem jMenuPrint = new JMenuItem();
    JMenu jMenuMove = new JMenu();
    JMenu jMenuEdit = new JMenu();
    JMenuItem jMenuUndo = new JMenuItem();
    JMenuItem jMenuCopy = new JMenuItem();
    JMenuItem jMenuCut = new JMenuItem();
    JMenuItem jMenuPaste = new JMenuItem();
    JMenuItem jMenuDelete = new JMenuItem();
    JMenuItem jMenuInputDate = new JMenuItem();
    JMenuItem jMenuFind = new JMenuItem();
    JMenuItem jMenuFindAgain = new JMenuItem();
    JMenuItem jMenuGoto = new JMenuItem();
    JMenu jMenuOptions = new JMenu();
    JMenuItem jMenuFont = new JMenuItem();
    JCheckBoxMenuItem jMenuWordWrap = new JCheckBoxMenuItem();
    JMenuItem jMenuFilter = new JMenuItem();
    JTabbedPane jTabbedPane = new JTabbedPane();
    JTextArea jCurrentTextArea;
    JMenu jMenuLookAndFeel = new JMenu();
    JMenuItem jMenuWindowsLook = new JMenuItem();
    JMenuItem jMenuMotifLook = new JMenuItem();
    JMenuItem jMenuMetalLook = new JMenuItem();
    JMenuItem jMenuMacLook = new JMenuItem();
    JMenuItem jMenuItem2 = new JMenuItem();
    JCheckBoxMenuItem jMenuAllWordWrap = new JCheckBoxMenuItem();
    JMenuItem jMenuAllFonts = new JMenuItem();
    JMenuItem jMenuItem5 = new JMenuItem();
    JMenuItem jMenuPageFormat = new JMenuItem();
    JButton jNewButton = new JButton();
    JButton jPrintButton = new JButton();
    JButton jCloseButton = new JButton();
    JButton jCopyButton = new JButton();
    JButton jCutButton = new JButton();
    JButton jPasteButton = new JButton();
    JButton jFindButton = new JButton();
    JButton jFindNextButton = new JButton();
    JButton jGotoButton = new JButton();
    JButton jFontButton = new JButton();
    JButton jUndoButton = new JButton();
    JPanel jKeyPanel = new JPanel();
    JLabel jCapsLabel = new JLabel();
    JLabel jNumLabel = new JLabel();
    JLabel jInsLabel = new JLabel();
    GridBagLayout gridBagLayout1 = new GridBagLayout();
    JMenuItem jMenuRedo = new JMenuItem();
    JMenuItem jMenuHelpIndex = new JMenuItem();
    JMenuItem jMenuHelpKeys = new JMenuItem();


    /**
     *  Constructor of the frame
     */
    public EditorFrame()
    {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try
        {
            jbInit();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }


    /**
     *  Sets the OverwriteMode attribute of the EditorFrame object<br>
     *  This updates the corresponding jInsLabel field.
     *
     * @param  aValue  The new OverwriteMode value
     */
    private void setOverwriteMode(boolean aValue)
    {
        overwriteMode = aValue;
        if (overwriteMode)
        {
            jInsLabel.setText("ber");
        }
        else
        {
            jInsLabel.setText("Einfg");
        }
    }


    /**
     *  Gets the OverwriteMode attribute
     *
     * @return    The OverwriteMode value
     */
    private boolean isOverwriteMode()
    {
        return overwriteMode;
    }


    /**
     *  Gets a new font by a FontChooser dialog
     *
     * @param  aFont  Current font
     * @return        New font
     */
    private Font getFont(Font aFont)
    {
        FontChooser dlg = new FontChooser(this);
        Dimension dlgSize = dlg.getPreferredSize();
        Dimension frmSize = getSize();
        Point loc = getLocation();
        dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
        dlg.setModal(true);
        boolean result = dlg.showDialog(aFont);
        if (result)
        {
            return dlg.getFont();
        }
        else
        {
            return null;
        }
    }


    /**
     *  Initialization of the gui components
     */
    private void jbInit() throws Exception
    {
        this.setSize(new Dimension(445, 300));
        String fn = null;
        try
        {
            fn = System.getProperty("IniFile");
        }
        catch (AccessControlException ex)
        {
            // Tritt unter Webstart auf, wenn nicht gengend Rechte vorhanden sind.
            System.out.println("Peters Multi-Editor hat nicht gengend Rechte, um auf ihrer Machine zu laufen!");
            ex.printStackTrace();
        }
        if (fn != null)
        {
            iniFileName = fn;
        }
        newIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/new.gif"));
        openIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/open.gif"));
        saveIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/save.gif"));
        saveasIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/saveas.gif"));
        closeIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/close.gif"));
        printIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/print.gif"));
        copyIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/copy.gif"));
        cutIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/cut.gif"));
        pasteIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/paste.gif"));
        findIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/find.gif"));
        findnextIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/findnext.gif"));
        gotoIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/goto.gif"));
        undoIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/undo.gif"));
        fontIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/font.gif"));
        helpIcon = new ImageIcon(getClass().getResource("/de/pschuett/images/help.gif"));
        logoIcon = new ImageIcon(getClass().getResource("logo.gif"));
        //setIconImage(Toolkit.getDefaultToolkit().createImage(EditorFrame.class.getResource("[Ihr Symbol]")));
        contentPane = (JPanel)this.getContentPane();
        contentPane.setLayout(borderLayout1);
        this.setTitle("Peters Multi-Editor");
        this.setIconImage(logoIcon.getImage());
        statusBar.setDisplayedMnemonic('0');
        statusBar.setHorizontalAlignment(SwingConstants.LEFT);
        statusBar.setHorizontalTextPosition(SwingConstants.LEFT);
        statusBar.setText(" Peters Multi-Editor");
        jMenuFile.setMnemonic('D');
        jMenuFile.setText("Datei");
        jMenuFileExit.setMnemonic('B');
        jMenuFileExit.setText("Beenden");
        jMenuFileExit.addActionListener(
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuFileExit_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuFileExit_actionPerformed(e);
                }
            });
        jMenuHelp.setMnemonic('H');
        jMenuHelp.setText("Hilfe");
        jMenuHelpAbout.setMnemonic('I');
        jMenuHelpAbout.setText("Info...");
        jMenuHelpAbout.addActionListener(
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuHelpAbout_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuHelpAbout_actionPerformed(e);
                }
            });
        ActionListener helpActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuHelp_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuHelp_actionPerformed(e);
                }
            };
        jMenuHelpIndex.addActionListener(helpActionListener);
        jMenuHelpKeys.addActionListener(helpActionListener);
        jHelpButton.addActionListener(helpActionListener);
        jOpenButton.setIcon(openIcon);
        ActionListener openActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jOpenButton_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jOpenButton_actionPerformed(e);
                }
            };
        jOpenButton.addActionListener(openActionListener);
        jOpenButton.setMaximumSize(new Dimension(29, 29));
        jOpenButton.setMinimumSize(new Dimension(29, 29));
        jOpenButton.setPreferredSize(new Dimension(29, 29));
        jOpenButton.setRequestFocusEnabled(false);
        jOpenButton.setToolTipText("Datei ffnen");
        jSaveButton.setIcon(saveIcon);
        ActionListener saveActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jSaveButton_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jSaveButton_actionPerformed(e);
                }
            };
        jSaveButton.addActionListener(saveActionListener);
        jSaveButton.setMaximumSize(new Dimension(29, 29));
        jSaveButton.setMinimumSize(new Dimension(29, 29));
        jSaveButton.setPreferredSize(new Dimension(29, 29));
        jSaveButton.setRequestFocusEnabled(false);
        jSaveButton.setToolTipText("Datei sichern");
        jHelpButton.setIcon(helpIcon);
        jSaveAsButton.setMaximumSize(new Dimension(29, 29));
        jSaveAsButton.setMinimumSize(new Dimension(29, 29));
        jSaveAsButton.setPreferredSize(new Dimension(29, 29));
        jSaveAsButton.setRequestFocusEnabled(false);
        jSaveAsButton.setToolTipText("Datei sichern unter");
        jSaveAsButton.setIcon(saveasIcon);
        jSaveAsButton.setText("");
        ActionListener saveasActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jSaveAsButton_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jSaveAsButton_actionPerformed(e);
                }
            };
        jSaveAsButton.addActionListener(saveasActionListener);
        jHelpButton.setMaximumSize(new Dimension(29, 29));
        jHelpButton.setMinimumSize(new Dimension(29, 29));
        jHelpButton.setPreferredSize(new Dimension(29, 29));
        jHelpButton.setRequestFocusEnabled(false);
        jHelpButton.setToolTipText("Hilfe");
        jMenuFileSave.setIcon(saveIcon);
        jMenuFileSave.setMnemonic('S');
        jMenuFileSave.setText("Sichern");
        jMenuFileSave.addActionListener(saveActionListener);
        jMenuFileOpen.setIcon(openIcon);
        jMenuFileOpen.setMnemonic('F');
        jMenuFileOpen.setText("ffnen...");
        jMenuFileOpen.addActionListener(openActionListener);
        jMenuFileSaveAs.setIcon(saveasIcon);
        jMenuFileSaveAs.setMnemonic('U');
        jMenuFileSaveAs.setText("Sichern unter...");
        jMenuFileSaveAs.addActionListener(saveasActionListener);
        ActionListener printActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuPrint_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuPrint_actionPerformed(e);
                }
            };
        jMenuPrint.addActionListener(printActionListener);
        jPrintButton.addActionListener(printActionListener);
        jMenuPageFormat.addActionListener(
            new java.awt.event.ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuPageFormat_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuPageFormat_actionPerformed(e);
                }
            });
        statusPane.setLayout(gridBagLayout);
        jXPosLabel.setToolTipText("Spalte");
        jXPosLabel.setText("0");
        jYPosLabel.setToolTipText("Zeile");
        jYPosLabel.setText("0");
        jSizeLabel.setToolTipText("Dateigre");
        jSizeLabel.setText("0");
        ActionListener closeActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuClose_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuClose_actionPerformed(e);
                }
            };
        jMenuClose.addActionListener(closeActionListener);
        jCloseButton.addActionListener(closeActionListener);
        jTabbedPane.addChangeListener(
            new javax.swing.event.ChangeListener()
            {
                /**
                 * {@link EditorFrame#jTabbedState_actionPerformed}
                 */
                public void stateChanged(ChangeEvent e)
                {
                    jTabbedPane_stateChanged(e);
                }
            });
        DropTarget target = new DropTarget(jTabbedPane, new EditorDropListener());
        jMenuLookAndFeel.setMnemonic('E');
        jMenuLookAndFeel.setText("Erscheinungsbild");
        ActionListener lookActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuLook_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuLook_actionPerformed(e);
                }
            };
        jMenuWindowsLook.setText("Windows");
        jMenuWindowsLook.addActionListener(lookActionListener);
        jMenuMotifLook.setText("Motif");
        jMenuMotifLook.addActionListener(lookActionListener);
        jMenuMetalLook.setText("Metal");
        jMenuMetalLook.addActionListener(lookActionListener);
        jMenuMacLook.setText("Mac");
        jMenuMacLook.addActionListener(lookActionListener);
        ActionListener undoActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuUndo_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuUndo_actionPerformed(e);
                }
            };
        jMenuUndo.addActionListener(undoActionListener);
        jUndoButton.addActionListener(undoActionListener);
        jMenuRedo.addActionListener(
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuRedo_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuRedo_actionPerformed(e);
                }
            });
        jMenuUndo.addActionListener(undoActionListener);
        jUndoButton.addActionListener(undoActionListener);
        ActionListener copyActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuCopy_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuCopy_actionPerformed(e);
                }
            };
        jMenuCopy.addActionListener(copyActionListener);
        jCopyButton.addActionListener(copyActionListener);
        ActionListener cutActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuCut_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuCut_actionPerformed(e);
                }
            };
        jMenuCut.addActionListener(cutActionListener);
        jCutButton.addActionListener(cutActionListener);
        ActionListener pasteActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuPaste_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuPaste_actionPerformed(e);
                }
            };
        jMenuPaste.addActionListener(pasteActionListener);
        jPasteButton.addActionListener(pasteActionListener);
        jMenuDelete.addActionListener(
            new java.awt.event.ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuDelete_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuDelete_actionPerformed(e);
                }
            });
        jMenuInputDate.addActionListener(
            new java.awt.event.ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuInputDate_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuInputDate_actionPerformed(e);
                }
            });
        jMenuFont.addActionListener(
            new java.awt.event.ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuFont_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuFont_actionPerformed(e);
                }
            });
        jMenuWordWrap.addActionListener(
            new java.awt.event.ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuWordWrap_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuWordWrap_actionPerformed(e);
                }
            });
        jMenuOptions.addMenuListener(
            new javax.swing.event.MenuListener()
            {
                /**
                 * {@link EditorFrame#jMenuOptions_menuSelected}
                 */
                public void menuSelected(MenuEvent e)
                {
                    jMenuOptions_menuSelected(e);
                }

                public void menuDeselected(MenuEvent e)
                {
                }

                public void menuCanceled(MenuEvent e)
                {
                }
            });
        jMenuItem2.setEnabled(false);
        jMenuItem2.setText("--Aktuelle Datei--");
        jMenuItem5.setEnabled(false);
        jMenuItem5.setText("--Alle Dateien--");
        jMenuAllFonts.setIcon(fontIcon);
        jMenuAllFonts.setMnemonic('A');
        jMenuAllFonts.setText("Schriftart");
        ActionListener allFontsActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuAllFonts_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuAllFonts_actionPerformed(e);
                }
            };
        jMenuAllFonts.addActionListener(allFontsActionListener);
        jFontButton.addActionListener(allFontsActionListener);
        jMenuAllWordWrap.setText("Zeilenumbruch");
        jMenuAllWordWrap.setMnemonic('U');
        jMenuAllWordWrap.addActionListener(
            new java.awt.event.ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuAllWordWrap_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuAllWordWrap_actionPerformed(e);
                }
            });
        ActionListener newActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuFileNew_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuFileNew_actionPerformed(e);
                }
            };
        jMenuFileNew.addActionListener(newActionListener);
        jNewButton.addActionListener(newActionListener);
        jMenuFileNew.setIcon(newIcon);
        jMenuFileNew.setMnemonic('N');
        jMenuPrint.setIcon(printIcon);
        jMenuPrint.setMnemonic('D');
        jMenuPrint.setText("Drucken...");
        jMenuClose.setIcon(closeIcon);
        jMenuClose.setMnemonic('L');
        jMenuClose.setAccelerator(javax.swing.KeyStroke.getKeyStroke(115, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuEdit.setMnemonic('B');
        jMenuMove.setMnemonic('S');
        jMenuOptions.setMnemonic('O');
        jPosLabel.setToolTipText("Position in Gesamtdatei");
        ActionListener findActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuFind_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuFind_actionPerformed(e);
                }
            };
        jMenuFind.addActionListener(findActionListener);
        jFindButton.addActionListener(findActionListener);
        ActionListener findAgainActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuFindAgain_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuFindAgain_actionPerformed(e);
                }
            };
        jMenuFindAgain.addActionListener(findAgainActionListener);
        jFindNextButton.addActionListener(findAgainActionListener);
        ActionListener gotoActionListener =
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuGoto_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuGoto_actionPerformed(e);
                }
            };
        jMenuGoto.addActionListener(gotoActionListener);
        jGotoButton.addActionListener(gotoActionListener);
        jMenuPageFormat.setText("Seite einrichten...");
        jMenuPageFormat.setMnemonic('T');
        jMenuCopy.setIcon(copyIcon);
        jMenuCopy.setMnemonic('K');
        jMenuCut.setIcon(cutIcon);
        jMenuCut.setMnemonic('A');
        jMenuPaste.setIcon(pasteIcon);
        jMenuPaste.setMnemonic('E');
        jMenuFind.setIcon(findIcon);
        jMenuFind.setMnemonic('S');
        jMenuFindAgain.setIcon(findnextIcon);
        jMenuFindAgain.setMnemonic('W');
        jMenuFindAgain.setAccelerator(javax.swing.KeyStroke.getKeyStroke(114, 0, false));
        jMenuGoto.setIcon(gotoIcon);
        jMenuGoto.setMnemonic('G');
        jNewButton.setMaximumSize(new Dimension(29, 29));
        jNewButton.setMinimumSize(new Dimension(29, 29));
        jNewButton.setPreferredSize(new Dimension(29, 29));
        jNewButton.setRequestFocusEnabled(false);
        jNewButton.setIcon(newIcon);
        jNewButton.setToolTipText("Neue Datei");
        jPrintButton.setMaximumSize(new Dimension(29, 29));
        jPrintButton.setMinimumSize(new Dimension(29, 29));
        jPrintButton.setPreferredSize(new Dimension(29, 29));
        jPrintButton.setRequestFocusEnabled(false);
        jPrintButton.setIcon(printIcon);
        jPrintButton.setToolTipText("Datei drucken");
        jCloseButton.setMaximumSize(new Dimension(29, 29));
        jCloseButton.setMinimumSize(new Dimension(29, 29));
        jCloseButton.setPreferredSize(new Dimension(29, 29));
        jCloseButton.setRequestFocusEnabled(false);
        jCloseButton.setIcon(closeIcon);
        jCloseButton.setToolTipText("Datei schlieen");
        jCopyButton.setMaximumSize(new Dimension(29, 29));
        jCopyButton.setMinimumSize(new Dimension(29, 29));
        jCopyButton.setPreferredSize(new Dimension(29, 29));
        jCopyButton.setRequestFocusEnabled(false);
        jCopyButton.setIcon(copyIcon);
        jCopyButton.setToolTipText("Kopieren");
        jCutButton.setMaximumSize(new Dimension(29, 29));
        jCutButton.setMinimumSize(new Dimension(29, 29));
        jCutButton.setPreferredSize(new Dimension(29, 29));
        jCutButton.setRequestFocusEnabled(false);
        jCutButton.setIcon(cutIcon);
        jCutButton.setToolTipText("Ausschneiden");
        jPasteButton.setMaximumSize(new Dimension(29, 29));
        jPasteButton.setMinimumSize(new Dimension(29, 29));
        jPasteButton.setPreferredSize(new Dimension(29, 29));
        jPasteButton.setRequestFocusEnabled(false);
        jPasteButton.setIcon(pasteIcon);
        jPasteButton.setToolTipText("Einfgen");
        jFindButton.setMaximumSize(new Dimension(29, 29));
        jFindButton.setMinimumSize(new Dimension(29, 29));
        jFindButton.setPreferredSize(new Dimension(29, 29));
        jFindButton.setRequestFocusEnabled(false);
        jFindButton.setIcon(findIcon);
        jFindButton.setToolTipText("Suchen/Ersetzen");
        jFindNextButton.setMaximumSize(new Dimension(29, 29));
        jFindNextButton.setMinimumSize(new Dimension(29, 29));
        jFindNextButton.setPreferredSize(new Dimension(29, 29));
        jFindNextButton.setRequestFocusEnabled(false);
        jFindNextButton.setIcon(findnextIcon);
        jFindNextButton.setToolTipText("Weitersuchen");
        jGotoButton.setMaximumSize(new Dimension(29, 29));
        jGotoButton.setMinimumSize(new Dimension(29, 29));
        jGotoButton.setPreferredSize(new Dimension(29, 29));
        jGotoButton.setRequestFocusEnabled(false);
        jGotoButton.setIcon(gotoIcon);
        jGotoButton.setToolTipText("Gehe zu");
        jFontButton.setMaximumSize(new Dimension(29, 29));
        jFontButton.setMinimumSize(new Dimension(29, 29));
        jFontButton.setPreferredSize(new Dimension(29, 29));
        jFontButton.setRequestFocusEnabled(false);
        jFontButton.setIcon(fontIcon);
        jFontButton.setToolTipText("Schriftart fr alle angezeigten Dateien einstellen");
        jUndoButton.setMaximumSize(new Dimension(29, 29));
        jUndoButton.setMinimumSize(new Dimension(29, 29));
        jUndoButton.setPreferredSize(new Dimension(29, 29));
        jUndoButton.setRequestFocusEnabled(false);
        jUndoButton.setIcon(undoIcon);
        jUndoButton.setToolTipText("Rckgngig");
        jKeyPanel.setMaximumSize(new Dimension(100, 15));
        jKeyPanel.setMinimumSize(new Dimension(100, 15));
        jKeyPanel.setPreferredSize(new Dimension(100, 15));
        jKeyPanel.setLayout(gridBagLayout1);
        jCapsLabel.setFont(new java.awt.Font("Dialog", 0, 10));
        jCapsLabel.setMaximumSize(new Dimension(30, 10));
        jCapsLabel.setMinimumSize(new Dimension(30, 10));
        jCapsLabel.setPreferredSize(new Dimension(30, 10));
        jCapsLabel.setToolTipText("");
        jCapsLabel.setHorizontalAlignment(SwingConstants.CENTER);
        jCapsLabel.setHorizontalTextPosition(SwingConstants.CENTER);
        jCapsLabel.setText("");
        jNumLabel.setFont(new java.awt.Font("Dialog", 0, 10));
        jNumLabel.setMaximumSize(new Dimension(30, 10));
        jNumLabel.setMinimumSize(new Dimension(30, 10));
        jNumLabel.setPreferredSize(new Dimension(30, 10));
        jNumLabel.setHorizontalAlignment(SwingConstants.CENTER);
        jNumLabel.setText("");
        jInsLabel.setFont(new java.awt.Font("Dialog", 0, 10));
        jInsLabel.setMaximumSize(new Dimension(30, 10));
        jInsLabel.setMinimumSize(new Dimension(30, 10));
        jInsLabel.setPreferredSize(new Dimension(30, 10));
        jInsLabel.setHorizontalAlignment(SwingConstants.CENTER);
        jInsLabel.setText("");
        jMenuRedo.setText("Wiederholen");
        jMenuRedo.setAccelerator(javax.swing.KeyStroke.getKeyStroke(89, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuRedo.setMnemonic('W');
        jMenuUndo.setMnemonic('R');
        jMenuUndo.setAccelerator(javax.swing.KeyStroke.getKeyStroke(90, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuDelete.setMnemonic('L');
        jMenuInputDate.setMnemonic('D');
        jMenuFont.setMnemonic('S');
        jMenuWordWrap.setMnemonic('Z');
        jMenuFilter.setMnemonic('F');
        jMenuHelpIndex.setText("Index");
        jMenuHelpIndex.setIcon(helpIcon);
        jMenuHelpIndex.setMnemonic('I');
        jMenuHelpIndex.setAccelerator(javax.swing.KeyStroke.getKeyStroke(112, 0, false));
        jMenuHelpKeys.setText("Tastatur");
        jMenuHelpKeys.setMnemonic('T');
        jPosLabel.setText("0");
        jMenuFileNew.setText("Neu");
        jMenuClose.setText("Schlieen");
        jMenuPrint.setText("Drucken...");
        jMenuEdit.setText("Bearbeiten");
        jMenuUndo.setText("Rckgngig");
        jMenuCopy.setText("Kopieren");
        jMenuCut.setText("Ausschneiden");
        jMenuPaste.setText("Einfgen");
        jMenuDelete.setText("Lschen");
        jMenuInputDate.setText("Datum");
        jMenuMove.setText("Suchen");
        jMenuFind.setText("Suchen / Ersetzen...");
        jMenuFindAgain.setText("Weitersuchen");
        jMenuGoto.setText("Gehe zu...");
        jMenuOptions.setText("Optionen");
        jMenuFont.setText("Schriftart");
        jMenuWordWrap.setText("Zeilenumbruch");
        jMenuFilter.setText("Filter...");
        jMenuFilter.addActionListener(
            new ActionListener()
            {
                /**
                 * {@link EditorFrame#jMenuFilter_actionPerformed}
                 */
                public void actionPerformed(ActionEvent e)
                {
                    jMenuFilter_actionPerformed(e);
                }
            });
        jToolBar.add(jNewButton, null);
        jToolBar.add(jOpenButton, null);
        jToolBar.add(jSaveButton, null);
        jToolBar.add(jSaveAsButton, null);
        jToolBar.add(jPrintButton, null);
        jToolBar.add(jCloseButton, null);
        jToolBar.add(jCopyButton, null);
        jToolBar.add(jCutButton, null);
        jToolBar.add(jPasteButton, null);
        jToolBar.add(jFindButton, null);
        jToolBar.add(jFindNextButton, null);
        jToolBar.add(jGotoButton, null);
        jToolBar.add(jUndoButton, null);
        jToolBar.add(jFontButton, null);
        jToolBar.add(jHelpButton, null);
        jMenuFile.add(jMenuFileNew);
        jMenuFile.add(jMenuFileOpen);
        jMenuFile.add(jMenuFileSave);
        jMenuFile.add(jMenuFileSaveAs);
        jMenuFile.addSeparator();
        jMenuFile.add(jMenuPrint);
        jMenuFile.add(jMenuPageFormat);
        jMenuFile.addSeparator();
        jMenuFile.add(jMenuClose);
        jMenuFile.addSeparator();
        jMenuFile.add(jMenuFileExit);
        jMenuHelp.add(jMenuHelpIndex);
        jMenuHelp.add(jMenuHelpKeys);
        jMenuHelp.addSeparator();
        jMenuHelp.add(jMenuHelpAbout);
        jMenuBar1.add(jMenuFile);
        jMenuBar1.add(jMenuEdit);
        jMenuBar1.add(jMenuMove);
        jMenuBar1.add(jMenuOptions);
        jMenuBar1.add(jMenuHelp);
        this.setJMenuBar(jMenuBar1);
        contentPane.add(jToolBar, BorderLayout.NORTH);
        contentPane.add(statusPane, BorderLayout.SOUTH);
        statusPane.add(jXPosLabel, new GridBagConstraints(0, 0, 1, 1, 100.0, 100.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        statusPane.add(jYPosLabel, new GridBagConstraints(1, 0, 1, 1, 100.0, 100.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        statusPane.add(jPosLabel, new GridBagConstraints(2, 0, 1, 1, 100.0, 100.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        statusPane.add(jSizeLabel, new GridBagConstraints(3, 0, 1, 1, 100.0, 100.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        statusPane.add(statusBar, new GridBagConstraints(4, 0, 1, 1, 100.0, 100.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        statusPane.add(jKeyPanel, new GridBagConstraints(5, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        jKeyPanel.add(jCapsLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 10.0
                , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        jKeyPanel.add(jInsLabel, new GridBagConstraints(1, 0, 1, 1, 0.0, 10.0
                , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        jKeyPanel.add(jNumLabel, new GridBagConstraints(2, 0, 1, 1, 0.0, 10.0
                , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        contentPane.add(jTabbedPane, BorderLayout.CENTER);
        jMenuEdit.add(jMenuUndo);
        jMenuEdit.add(jMenuRedo);
        jMenuEdit.addSeparator();
        jMenuEdit.add(jMenuCopy);
        jMenuEdit.add(jMenuCut);
        jMenuEdit.add(jMenuPaste);
        jMenuEdit.add(jMenuDelete);
        jMenuEdit.addSeparator();
        jMenuEdit.add(jMenuInputDate);
        jMenuMove.add(jMenuFind);
        jMenuMove.add(jMenuFindAgain);
        jMenuMove.addSeparator();
        jMenuMove.add(jMenuGoto);
        jMenuOptions.add(jMenuItem2);
        jMenuOptions.add(jMenuFont);
        jMenuOptions.add(jMenuWordWrap);
        jMenuOptions.add(jMenuItem5);
        jMenuOptions.add(jMenuAllFonts);
        jMenuOptions.add(jMenuAllWordWrap);
        jMenuOptions.addSeparator();
        jMenuOptions.add(jMenuFilter);
        jMenuOptions.addSeparator();
        jMenuOptions.add(jMenuLookAndFeel);
        jMenuLookAndFeel.add(jMenuWindowsLook);
        jMenuLookAndFeel.add(jMenuMacLook);
        jMenuLookAndFeel.add(jMenuMotifLook);
        jMenuLookAndFeel.add(jMenuMetalLook);
        setOverwriteMode(overwriteMode);
        loadIni(new File(iniFileName));
        addNewEditor();
        Toolkit.getDefaultToolkit().addAWTEventListener(
            new AWTEventListener()
            {
                /**
                 *  Event handler to change the shown editor by Ctrl-Tab and Shift-Ctrl-Tab
                 */
                public void eventDispatched(AWTEvent event)
                {
                    KeyEvent ev = (KeyEvent)event;
                    if (ev.getID() == ev.KEY_PRESSED)
                    {
                        if ((ev.getKeyCode() == ev.VK_TAB) && ((ev.getModifiers() & ev.CTRL_MASK) != 0))
                        {
                            if ((ev.getModifiers() & ev.SHIFT_MASK) != 0)
                            {
                                int ii = jTabbedPane.getSelectedIndex() - 1;
                                if (ii < 0)
                                {
                                    ii = jTabbedPane.getTabCount() - 1;
                                }
                                jTabbedPane.setSelectedIndex(ii);
                            }
                            else
                            {
                                jTabbedPane.setSelectedIndex((jTabbedPane.getSelectedIndex() + 1) % jTabbedPane.getTabCount());
                            }
                        }
                        jCurrentTextArea.grabFocus();
                    }
                }
            }, AWTEvent.KEY_EVENT_MASK);
    }


    /**
     *  Method which creates the new editors<br>
     *  Every editor is initialized here.
     *
     * @return    New editor
     */
    private JTextArea addNewEditor()
    {
        if (jCurrentTextArea != null)
        {
            jCurrentTextArea.getDocument().putProperty("modified", new Boolean(currentModified));
        }
        JScrollPane jScrollPane = new JScrollPane();
        jCurrentTextArea = new JTextArea();
        if (defaultFont != null)
        {
            jCurrentTextArea.setFont(defaultFont);
        }
        if (defaultFont == null)
        {
            defaultFont = jCurrentTextArea.getFont();
        }
        jCurrentTextArea.setLineWrap(allWordWrap);
        jCurrentTextArea.setWrapStyleWord(allWordWrap);
        DropTarget target = new DropTarget(jCurrentTextArea, new EditorDropListener());
        jScrollPane.getViewport().add(jCurrentTextArea, null);
        jTabbedPane.addTab(UNNAMED, jScrollPane);
        jCurrentTextArea.addKeyListener(
            new java.awt.event.KeyAdapter()
            {
                /**
                 * {@link EditorFrame#jTextArea_keyReleased}
                 */
                public void keyReleased(KeyEvent e)
                {
                    jTextArea_keyReleased(e);
                }


                /**
                 * {@link EditorFrame#jTextArea_keyPressed}
                 */
                public void keyPressed(KeyEvent e)
                {
                    jTextArea_keyPressed(e);
                }
            });
        jCurrentTextArea.addCaretListener(
            new javax.swing.event.CaretListener()
            {
                /**
                 * {@link EditorFrame#jTextArea_caretUpdate}
                 */
                public void caretUpdate(CaretEvent e)
                {
                    jTextArea_caretUpdate(e);
                }
            });
        jTabbedPane.setSelectedIndex(jTabbedPane.getTabCount() - 1);
        jCurrentTextArea.getDocument().addDocumentListener(new EditorDocumentListener());
        UndoManager undoManager = new UndoManager();
        jCurrentTextArea.getDocument().putProperty("UndoManager", undoManager);
        jCurrentTextArea.getDocument().addUndoableEditListener(undoManager);
        currentModified = false;
        return jCurrentTextArea;
    }


    /**
     *  This method closes the whole program, after closing every single editor
     *  by calling {@link #closeEditor}.
     *
     * @return    True if closing of the program is possible
     */
    private boolean closeAllEditors()
    {
        for (int ii = jTabbedPane.getTabCount() - 1; ii >= 0; ii--)
        {
            jTabbedPane.setSelectedIndex(ii);
            if (closeEditor() == JOptionPane.CANCEL_OPTION)
            {
                return false;
            }
        }
        saveIni(new File(iniFileName));
        System.exit(0);
        return true;
    }


    /**
     *  Shows the new values for the current caret position
     *
     * @param  sender  Current shown JTextArea
     */
    private void updatePositionInfo(JTextArea sender)
    {
        try
        {
            int caretPos = sender.getCaretPosition();
            int lineOfOffset = sender.getLineOfOffset(caretPos);
            int lineStart = sender.getLineStartOffset(lineOfOffset);
            jPosLabel.setText(new Integer(caretPos + 1).toString());
            jXPosLabel.setText(new Integer(caretPos - lineStart + 1).toString());
            jYPosLabel.setText(new Integer(lineOfOffset + 1).toString());
        }
        catch (Exception ex)
        {
            jXPosLabel.setText("err");
            jYPosLabel.setText("err");
            ex.printStackTrace();
        }
    }


    /**
     *  Shows the new size of an editor
     *
     * @param  size  size of the current editor
     */
    private void updateSizeLabel(int size)
    {
        jSizeLabel.setText(new Integer(size).toString());
    }


    /**
     *  Close a single editor<br>
     *  This method checks if something is modified and asks for saving.
     *
     * @return    True if it is closed.
     * @see  #closeAllEditors
     */
    private int closeEditor()
    {
        int result = JOptionPane.OK_OPTION;
        if (currentModified)
        {
            File file = (File)jCurrentTextArea.getDocument().getProperty("file");
            String fileName;
            if (file == null)
            {
                fileName = UNNAMED;
            }
            else
            {
                fileName = file.getAbsolutePath();
            }
            result = JOptionPane.showConfirmDialog(this, fileName + "\n"
                     + "wurde gendert.\n"
                     + "Wollen Sie die Datei vor dem Schlieen sichern?",
                    "Auswahl", JOptionPane.YES_NO_CANCEL_OPTION);
            switch (result)
            {
                case JOptionPane.CANCEL_OPTION:
                    break;
                case JOptionPane.YES_OPTION:
                    try
                    {
                        if (file == null)
                        {
                            jSaveAsButton_actionPerformed(null);
                        }
                        else
                        {
                            jCurrentTextArea.write(new FileWriter(file));
                        }
                    }
                    catch (Exception ex)
                    {
                        ex.printStackTrace();
                        if (JOptionPane.showConfirmDialog(this, fileName + "\n"
                                 + "Bei Schreiben der Datei ist ein Fehler aufgetreten!\n"
                                 + "Wollen Sie die Datei trotzdem schlieen?",
                                "Warnung", JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION)
                        {
                            return JOptionPane.CANCEL_OPTION;
                        }
                    }
            }
        }
        if (result != JOptionPane.CANCEL_OPTION)
        {
            jCurrentTextArea = null;
            jTabbedPane.remove(jTabbedPane.getSelectedIndex());
            if (jTabbedPane.getTabCount() == 0)
            {
                addNewEditor();
            }
            else if ((jTabbedPane.getSelectedIndex() < 0) && (jTabbedPane.getTabCount() > 0))
            {
                jTabbedPane.setSelectedIndex(0);
            }
            else
            {
                jTabbedPane_stateChanged(null);
            }
        }
        return result;
    }


    /**
     *  Loads a configuration in the beginning of the program<br>
     *  There two ways to find this file:
     *  <ul>
     *  <li>The file name is given by a enviroment property "IniFile"
     *  <li>Otherwise: The file PeteEdit.ini in the start directory is used.
     *  </ul>
     *
     * @param  file  configuration file
     * @see #saveIni(File file)
     */
    private void loadIni(File file)
    {
        try
        {
            IniFile ini = new IniFile(file);
            // Font
            int fontStyle = Font.PLAIN;
            if (ini.readBool("Font", "Bold", false))
            {
                fontStyle += Font.BOLD;
            }
            if (ini.readBool("Font", "Italic", false))
            {
                fontStyle += Font.ITALIC;
            }
            defaultFont = new Font(
                    ini.readString("Font", "Name", "Monospaced"),
                    fontStyle,
                    ini.readInteger("Font", "Size", 12));
            // Window
            Rectangle r = this.getBounds();
            this.setBounds(
                    ini.readInteger("Window", "Left", (int)r.getX()),
                    ini.readInteger("Window", "Top", (int)r.getY()),
                    ini.readInteger("Window", "Width", (int)r.getWidth()),
                    ini.readInteger("Window", "Height", (int)r.getHeight())
                    );
            //Filter
            filterString = ini.readString("Files", "Filter", "Text-Dateien (*.txt)|*.txt|Rest (*.*)|*.*|");
            filters = DefaultFileFilter.parseFileFilterString(filterString);

            // printer Options
            //defaultPageFormat = PrinterJob.getPrinterJob().defaultPage();
            defaultPageFormat = new PageFormat();
            defaultPageFormat.setOrientation(ini.readInteger("Printer", "orientation", defaultPageFormat.getOrientation()));
            Paper paper = defaultPageFormat.getPaper();
            paper.setImageableArea(
                    ini.read("Printer", "ImageLeft", paper.getImageableX()),
                    ini.read("Printer", "ImageTop", paper.getImageableY()),
                    ini.read("Printer", "ImageWidth", paper.getImageableWidth()),
                    ini.read("Printer", "ImageHeight", paper.getImageableHeight())
                    );
            paper.setSize(
                    ini.read("Printer", "Width", paper.getWidth()),
                    ini.read("Printer", "Height", paper.getHeight())
                    );
            defaultPageFormat.setPaper(paper);

            // other Options
            allWordWrap = ini.readBool("Options", "WordWrap", allWordWrap);
            try
            {
                UIManager.setLookAndFeel(ini.readString("Options", "LookAndFeel", UIManager.getLookAndFeel().getClass().getName()));
                SwingUtilities.updateComponentTreeUI(this);
            }
            catch (Exception ex)
            {
                ex.printStackTrace();
                JOptionPane.showMessageDialog(this, "Fehler beim Einstellen des Erscheinungsbilds!\n"
                         + "Wahrscheinlich ist das in der Konfigurationsdatei eingetragene Erscheinungsbild "
                         + "auf der aktuellen Platform nicht verfgbar.\n"
                         + "Es wird nun ein Default-Erscheinungsbild verwendet.");
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }


    /**
     *  Saves the program configuration in a configuration file.
     *
     * @param  file  configuration file
     * @see #loadIni(File file)
     */
    private void saveIni(File file)
    {
        try
        {
            IniFile ini = new IniFile(file);
            // Font
            ini.writeString("Font", "Name", defaultFont.getName());
            ini.writeBool("Font", "Bold", (defaultFont.getStyle() & Font.BOLD) != 0);
            ini.writeBool("Font", "Italic", (defaultFont.getStyle() & Font.ITALIC) != 0);
            ini.writeInteger("Font", "Size", defaultFont.getSize());
            // Window
            this.setState(Frame.NORMAL);
            Rectangle r = this.getBounds();
            ini.writeInteger("Window", "Left", (int)r.getX());
            ini.writeInteger("Window", "Top", (int)r.getY());
            ini.writeInteger("Window", "Width", (int)r.getWidth());
            ini.writeInteger("Window", "Height", (int)r.getHeight());
            //Filter
            ini.writeString("Files", "Filter", filterString);

            // printer Options
            ini.writeInteger("Printer", "orientation", defaultPageFormat.getOrientation());
            Paper paper = defaultPageFormat.getPaper();
            ini.write("Printer", "ImageLeft", paper.getImageableX());
            ini.write("Printer", "ImageTop", paper.getImageableY());
            ini.write("Printer", "ImageWidth", paper.getImageableWidth());
            ini.write("Printer", "ImageHeight", paper.getImageableHeight());
            ini.write("Printer", "Width", paper.getWidth());
            ini.write("Printer", "Height", paper.getHeight());

            // other Options
            ini.writeBool("Options", "WordWrap", allWordWrap);
            ini.writeString("Options", "LookAndFeel", UIManager.getLookAndFeel().getClass().getName());
            ini.flush();
            System.out.println("Ini-file: " + file.getAbsolutePath());
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }


    /**
     *  Event handler for opening a new editor
     */
    void jMenuFileNew_actionPerformed(ActionEvent e)
    {
        addNewEditor();
        jTabbedPane_stateChanged(null);
    }


    /**
     *  Event handler for opening a file in a new editor
     */
    void jOpenButton_actionPerformed(ActionEvent e)
    {
        JFileChooser fc = new JFileChooser();
        fc.setMultiSelectionEnabled(true);
        fc.setFileHidingEnabled(true);
        FileFilter[] fils = fc.getChoosableFileFilters();
        if ((fils != null) && (fils.length > 0))
            fc.removeChoosableFileFilter(fils[0]);
        for (int ii = 0; ii < filters.length; ii++)
        {
            fc.addChoosableFileFilter(filters[ii]);
        }
        if (defaultFilter != null)
        {
            fc.setFileFilter(defaultFilter);
        }
        else
        {
            fc.setFileFilter(filters[0]);
        }
        int returnVal = fc.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            File[] files = fc.getSelectedFiles();
            if (files.length == 0)
            {
                openFile(fc.getSelectedFile());
            }
            else
            {
                for (int ii = 0; ii < files.length; ii++)
                {
                    openFile(files[ii]);
                }
            }
        }
    }


    /**
     *  Event handler for saving the current editor<br>
     *  If the file has no name the
     *  {@link #jSaveAsButton_actionPerformed(java.awt.event.ActionEvent e)} method
     *  is called.
     */
    void jSaveButton_actionPerformed(ActionEvent e)
    {
        File file = (File)jCurrentTextArea.getDocument().getProperty("file");
        if (file == null)
        {
            jSaveAsButton_actionPerformed(e);
            return;
        }
        try
        {
            jCurrentTextArea.write(new FileWriter(file));
            currentModified = false;
        }
        catch (Exception ex)
        {
            JOptionPane.showMessageDialog(this, "Fehler beim Sichern der Datei\n"
                     + file.getAbsolutePath());
        }

    }


    /**
     *  Event handler for saving the current editor with a new name.
     */
    void jSaveAsButton_actionPerformed(ActionEvent e)
    {
        JFileChooser fc = new JFileChooser();
        fc.setFileHidingEnabled(true);
        FileFilter[] fils = fc.getChoosableFileFilters();
        if ((fils != null) && (fils.length > 0))
            fc.removeChoosableFileFilter(fils[0]);
        for (int ii = 0; ii < filters.length; ii++)
        {
            fc.addChoosableFileFilter(filters[ii]);
        }
        if (defaultFilter != null)
        {
            fc.setFileFilter(defaultFilter);
        }
        else
        {
            fc.setFileFilter(filters[0]);
        }
        int returnVal = fc.showSaveDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            File file = null;
            try
            {
                file = fc.getSelectedFile();
                jCurrentTextArea.write(new FileWriter(file));
                jCurrentTextArea.getDocument().putProperty("file", file);
                statusBar.setText(file.getAbsolutePath());
                jTabbedPane.setTitleAt(jTabbedPane.getSelectedIndex(), file.getName());
                currentModified = false;
            }
            catch (Exception ex)
            {
                JOptionPane.showMessageDialog(this, "Fehler beim Sichern der Datei\n"
                         + file.getAbsolutePath());
                ex.printStackTrace();
            }
        }
    }


    /**
     *  Method for open a file into a new editor
     *
     * @param  aFile  file to open
     */
    void openFile(File aFile)
    {
        JTextArea textArea = null;
        if ((jCurrentTextArea.getText().length() == 0)
                 && (jTabbedPane.getTitleAt(jTabbedPane.getSelectedIndex()).equals(UNNAMED)))
        {
            textArea = jCurrentTextArea;
        }
        else
        {
            textArea = addNewEditor();
        }
        try
        {
            // the following command is used analogous to the source code of JTextComponent#read
            textArea.getUI().getEditorKit(textArea).read(new FileReader(aFile), textArea.getDocument(), 0);
            ((UndoManager)textArea.getDocument().getProperty("UndoManager")).discardAllEdits();
            textArea.getDocument().putProperty("file", aFile);
            textArea.setCaretPosition(0);
            currentModified = false;
            jTabbedPane.setTitleAt(jTabbedPane.getSelectedIndex(), aFile.getName());
            jTabbedPane_stateChanged(null);
        }
        catch (Exception ex)
        {
            JOptionPane.showMessageDialog(this, "Fehler beim Laden der Datei\n"
                     + aFile.getAbsolutePath());
        }
    }


    /**
     *  Event handler to get changes in the caret position
     *  @see #updatePositionInfo(JTextArea sender)
     */
    void jTextArea_caretUpdate(CaretEvent e)
    {
        updatePositionInfo((JTextArea)e.getSource());
    }


    /**
     *  Key event handler to register the insert/overwrite modus
     */
    void jTextArea_keyReleased(KeyEvent e)
    {
        if ((e.getKeyCode() == e.VK_INSERT) && (e.getModifiers() == 0))
        {
            setOverwriteMode(!isOverwriteMode());
            return;
        }
    }


    /**
     *  Print menu item handler<br>
     *  If this event is fired by jMenuPrint then a dialog is shown.<br>
     *  If this event is fired by jPrintButton then the open editor is printed directly.
     *  @see EditorPrintable
     */
    void jMenuPrint_actionPerformed(ActionEvent e)
    {
        PrinterJob printJob = PrinterJob.getPrinterJob();
        printJob.setJobName(jTabbedPane.getTitleAt(jTabbedPane.getSelectedIndex()) + " : Peters Multi-Editor");
        printJob.setPrintable(new EditorPrintable(jCurrentTextArea), defaultPageFormat);
        if ((e.getSource() == jPrintButton) || printJob.printDialog())
        {
            try
            {
                setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                printJob.print();
            }
            catch (PrinterException ex)
            {
                ex.printStackTrace();
                JOptionPane.showMessageDialog(this, "Fehler beim Ausdruck!");
            }
            finally
            {
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            }
        }
    }


    /**
     *  Shows a native dialog to set the print page properties
     */
    void jMenuPageFormat_actionPerformed(ActionEvent e)
    {
        PrinterJob printJob = PrinterJob.getPrinterJob();
        defaultPageFormat = printJob.pageDialog(defaultPageFormat);
    }


    /**
     *  Event handler if an editor is closed (calls {@link #closeEditor()})
     */
    void jMenuClose_actionPerformed(ActionEvent e)
    {
        closeEditor();
    }


    /**
     *  Event handler if a new tab is selected<br>
     *  This method do the necessary initializations to show another editor.
     */
    void jTabbedPane_stateChanged(ChangeEvent e)
    {
        if (jCurrentTextArea != null)
        {
            jCurrentTextArea.getDocument().putProperty("modified", new Boolean(currentModified));
        }
        if ((jTabbedPane.getTabCount() == 0) || (jTabbedPane.getSelectedIndex() == -1))
        {
            return;
        }
        jCurrentTextArea = (JTextArea)((JScrollPane)jTabbedPane.getSelectedComponent()).getViewport().getComponent(0);
        currentModified = ((Boolean)jCurrentTextArea.getDocument().getProperty("modified")).booleanValue();
        updateSizeLabel(jCurrentTextArea.getDocument().getLength());
        updatePositionInfo(jCurrentTextArea);
        File file = (File)jCurrentTextArea.getDocument().getProperty("file");
        if (file != null)
        {
            statusBar.setText(file.getAbsolutePath());
        }
        else
        {
            statusBar.setText(UNNAMED);
        }
    }


    /**
     *  Event handler for changing the look and feel<br>
     *  This method handles all look-and-feel selection events
     */
    void jMenuLook_actionPerformed(ActionEvent e)
    {
        try
        {
            if (e.getSource() == jMenuWindowsLook)
            {
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
            }
            else if (e.getSource() == jMenuMacLook)
            {
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.mac.MacLookAndFeel");
            }
            else if (e.getSource() == jMenuMotifLook)
            {
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
            }
            else if (e.getSource() == jMenuMetalLook)
            {
                UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
            }
            SwingUtilities.updateComponentTreeUI(this);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }


    /**
     *  Event handler for an undo event
     *  @see #jMenuRedo_actionPerformed(ActionEvent e)
     */
    void jMenuUndo_actionPerformed(ActionEvent e)
    {
        UndoManager undoManager = (UndoManager)jCurrentTextArea.getDocument().getProperty("UndoManager");
        if (undoManager.canUndo())
        {
            undoManager.undo();
        }
    }


    /**
     *  Event handler for an redo event
     *  @see #jMenuUndo_actionPerformed(ActionEvent e)
     */
    void jMenuRedo_actionPerformed(ActionEvent e)
    {
        UndoManager undoManager = (UndoManager)jCurrentTextArea.getDocument().getProperty("UndoManager");
        if (undoManager.canRedo())
        {
            undoManager.redo();
        }
    }


    /**
     *  Event handler for the copy operation
     */
    void jMenuCopy_actionPerformed(ActionEvent e)
    {
        jCurrentTextArea.copy();
    }


    /**
     *  Event handler for the cut operation
     */
    void jMenuCut_actionPerformed(ActionEvent e)
    {
        jCurrentTextArea.cut();
    }


    /**
     *  Event handler for the paste operation
     */
    void jMenuPaste_actionPerformed(ActionEvent e)
    {
        jCurrentTextArea.paste();
    }


    /**
     *  Event handler for the delete operation
     */
    void jMenuDelete_actionPerformed(ActionEvent e)
    {
        int selStart = jCurrentTextArea.getSelectionStart();
        int selEnd = jCurrentTextArea.getSelectionEnd();
        try
        {
            jCurrentTextArea.getDocument().remove(selStart, selEnd - selStart);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }


    /**
     * Event handler for the input date operation
     */
    void jMenuInputDate_actionPerformed(ActionEvent e)
    {
        String dateStr = new SimpleDateFormat().format(new Date());
        int selStart = jCurrentTextArea.getSelectionStart();
        int selEnd = jCurrentTextArea.getSelectionEnd();
        try
        {
            jCurrentTextArea.getDocument().remove(selStart, selEnd - selStart);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
        jCurrentTextArea.insert(dateStr, selStart);
    }


    /**
     * Event handler for changing the font of one editor<br>
     * If this editor is close the changes of the font are lost.
     */
    void jMenuFont_actionPerformed(ActionEvent e)
    {
        Font font = getFont(jCurrentTextArea.getFont());
        if (font != null)
        {
            jCurrentTextArea.setFont(font);
        }
    }


    /**
     * Event handler for changing the word wrap setting of one editor<br>
     * If this editor is close the change of the word wrap setting is lost.
     *
     * @param  e  Description of Parameter
     */
    void jMenuWordWrap_actionPerformed(ActionEvent e)
    {
        boolean result = jCurrentTextArea.getWrapStyleWord();
        jCurrentTextArea.setWrapStyleWord(!result);
        jCurrentTextArea.setLineWrap(!result);
    }


    /**
     *  Event handler to show the selected state of jMenuWordWrap and jMenuAllWordWrap
     */
    void jMenuOptions_menuSelected(MenuEvent e)
    {
        jMenuWordWrap.setState(jCurrentTextArea.getWrapStyleWord());
        jMenuAllWordWrap.setState(allWordWrap);
    }


    /**
     * Event handler for changing the font of all editors<br>
     * These settings are saved into the configuration file after closing the program.
     */
    void jMenuAllFonts_actionPerformed(ActionEvent e)
    {
        Font font = getFont(defaultFont);
        if (font != null)
        {
            defaultFont = font;
            for (int ii = 0; ii < jTabbedPane.getTabCount(); ii++)
            {
                JTextArea textArea = (JTextArea)((JScrollPane)jTabbedPane.getComponentAt(ii)).getViewport().getComponent(0);
                textArea.setFont(defaultFont);
            }
        }
    }


    /**
     * Event handler for changing the word wrap style of all editors<br>
     * These settings are saved into the configuration file after closing the program.
     */
    void jMenuAllWordWrap_actionPerformed(ActionEvent e)
    {
        allWordWrap = !allWordWrap;
        for (int ii = 0; ii < jTabbedPane.getTabCount(); ii++)
        {
            JTextArea textArea = (JTextArea)((JScrollPane)jTabbedPane.getComponentAt(ii)).getViewport().getComponent(0);
            textArea.setWrapStyleWord(allWordWrap);
            textArea.setLineWrap(allWordWrap);
        }

    }


    /**
     * Event handler to open the find dialog
     */
    void jMenuFind_actionPerformed(ActionEvent e)
    {
        FindDialog dlg = new FindDialog(this);
        Dimension dlgSize = dlg.getPreferredSize();
        Dimension frmSize = getSize();
        Point loc = getLocation();
        ((EditorFindOptions)findOptions).setNewText();
        // if searching from beginning is used
        dlg.setFindOptions(findOptions);
        dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
        dlg.show();
    }


    /**
     * Event handler to do the find again action<br>
     * If still no search string is specified then
     * {@link #jMenuFind_actionPerformed(ActionEvent e)} is called.
     */
    void jMenuFindAgain_actionPerformed(ActionEvent e)
    {
        if ((findOptions.getSearchText() == null) || (findOptions.getSearchText().length() == 0))
        {
            jMenuFind_actionPerformed(e);
        }
        else
        {
            ((EditorFindOptions)findOptions).doAgain();
        }
    }


    /**
     *  Description of the Method
     *
     * @param  e  Description of Parameter
     */
    void jMenuGoto_actionPerformed(ActionEvent e)
    {
        String line = JOptionPane.showInputDialog(this, "Gehe zur Zeile:", "Zeilenauswahl", JOptionPane.QUESTION_MESSAGE);
        if ((line == null) || (line.length() == 0))
        {
            return;
        }
        try
        {
            int lineNo = new Integer(line).intValue();
            int pos = jCurrentTextArea.getLineStartOffset(lineNo - 1);
            jCurrentTextArea.setCaretPosition(pos);
        }
        catch (BadLocationException ex)
        {
            JOptionPane.showMessageDialog(this, "Zeile '" + line + "' existiert nicht!",
                    "Eingabefehler", JOptionPane.ERROR_MESSAGE);
        }
        catch (Exception ex)
        {
            JOptionPane.showMessageDialog(this, line + "\nist keine Zeilennummer!",
                    "Eingabefehler", JOptionPane.ERROR_MESSAGE);
        }
    }


    /**
     * Event handler to handle the overwrite mode and some other key properties
     */
    void jTextArea_keyPressed(KeyEvent e)
    {
        if ((e.getKeyChar() != e.CHAR_UNDEFINED) && isOverwriteMode())
        {
            String sel = jCurrentTextArea.getSelectedText();
            if ((sel == null) || (sel.length() == 0))
            {
                int pos = jCurrentTextArea.getCaretPosition();
                try
                {
                    if (!jCurrentTextArea.getText(pos, 1).equals("\n"))
                    {
                        jCurrentTextArea.select(pos, pos + 1);
                    }
                }
                catch (BadLocationException ex)
                {
                }
            }
        }
        else if ((e.getKeyCode() == e.VK_RIGHT) && e.isAltDown() && e.isControlDown())
        {
            int nn = jCurrentTextArea.getCaretPosition();
            String content = jCurrentTextArea.getText();
            while ((content.length() > nn) && ((content.charAt(nn) == ' ') || (content.charAt(nn) == '\t') || (content.charAt(nn) == '\n')))
            {
                nn++;
            }
            while ((content.length() > nn) && !((content.charAt(nn) == ' ') || (content.charAt(nn) == '\t') || (content.charAt(nn) == '\n')))
            {
                nn++;
            }
            jCurrentTextArea.setCaretPosition(nn);
        }
        else if ((e.getKeyCode() == e.VK_LEFT) && e.isAltDown() && e.isControlDown())
        {
            int nn = jCurrentTextArea.getCaretPosition() - 1;
            String content = jCurrentTextArea.getText();
            while ((nn > 0) && ((content.charAt(nn) == ' ') || (content.charAt(nn) == '\t') || (content.charAt(nn) == '\n')))
            {
                nn--;
            }
            while ((nn > 0) && !((content.charAt(nn) == ' ') || (content.charAt(nn) == '\t') || (content.charAt(nn) == '\n')))
            {
                nn--;
            }
            while ((nn > 0) && ((content.charAt(nn) == ' ') || (content.charAt(nn) == '\t') || (content.charAt(nn) == '\n')))
            {
                nn--;
            }
            if ((nn > 0) || (content.charAt(nn + 1) == ' '))
            {
                nn++;
            }
            jCurrentTextArea.setCaretPosition(nn);
        }
    }


    /**
     * Event handler which opens the filter dialog
     */
    void jMenuFilter_actionPerformed(ActionEvent e)
    {
        FilterDialog dlg = new FilterDialog(this);
        Dimension dlgSize = dlg.getPreferredSize();
        Dimension frmSize = getSize();
        Point loc = getLocation();
        dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
        dlg.setModal(true);
        if (dlg.showDialog(filterString))
        {
            filterString = dlg.getFilterString();
            filters = DefaultFileFilter.parseFileFilterString(filterString);
        }
    }


    /**
     * Overwritten to make the finishing of window possible
     */
    protected void processWindowEvent(WindowEvent e)
    {
        if (e.getID() == WindowEvent.WINDOW_CLOSING)
        {
            if (closeAllEditors())
            {
                super.processWindowEvent(e);
            }
        }
        else
        {
            super.processWindowEvent(e);
        }
    }


    /**
     *  Event handler to close the program
     */
    public void jMenuFileExit_actionPerformed(ActionEvent e)
    {
        closeAllEditors();
    }


    /**
     * Event handler to show help file
     */
    public void jMenuHelp_actionPerformed(ActionEvent e)
    {
        String url = getClass().getResource("/de/pschuett/editor/peteedit.html").toString();
        String title = "Hilfe fr " + this.getTitle();
        if (e.getSource() == jMenuHelpKeys)
            HelpFrame.showHTMLHelp(url+"#Tastatur", title, logoIcon, this);
        else
            HelpFrame.showHTMLHelp(url, title, logoIcon, this);
    }

    /**
     * Event handler to open the info dialog
     */
    public void jMenuHelpAbout_actionPerformed(ActionEvent e)
    {
        Infodialog dlg = new Infodialog(this);
        Dimension dlgSize = dlg.getPreferredSize();
        Dimension frmSize = getSize();
        Point loc = getLocation();
        dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
        dlg.setModal(true);
        dlg.setLogo(Toolkit.getDefaultToolkit().getImage(getClass().getResource("logo.gif")));
        dlg.show();
    }


    /**
     * Listener to handle the file drag and drop with other applications
     *
     * @author    ps
     */
    private class EditorDropListener implements DropTargetListener
    {
        /**
         * Returns true if drag mode is Copy or Move
         */
        public boolean isDragAcceptable(DropTargetDragEvent event)
        {
            return ((event.getDropAction() & DnDConstants.ACTION_COPY_OR_MOVE) != 0);
        }


        /**
         *  Returns true if Object is able to be dropped
         */
        public boolean isDropAcceptable(DropTargetDragEvent event)
        {
            return ((event.getDropAction() & DnDConstants.ACTION_COPY_OR_MOVE) != 0);
        }

        public void dragEnter(DropTargetDragEvent event)
        {
        }

        public void dragExit(DropTargetEvent event)
        {
        }

        public void dragOver(DropTargetDragEvent event)
        {
        }

        public void dropActionChanged(DropTargetDragEvent event)
        {
        }

        /**
         *  A file is dropped and a new editor is opened with this file.
         */
        public void drop(DropTargetDropEvent event)
        {
            event.acceptDrop(DnDConstants.ACTION_COPY);
            Transferable transferable = event.getTransferable();
            DataFlavor[] flavors = transferable.getTransferDataFlavors();
            for (int ii = 0; ii < flavors.length; ii++)
            {
                DataFlavor flavor = flavors[ii];
                Object data = null;
                try
                {
                    data = transferable.getTransferData(flavor);
                }
                catch (Exception ex)
                {
                    ex.printStackTrace();
                    JOptionPane.showMessageDialog(null, "Fehler beim Drag-and-Drop!");
                    event.dropComplete(false);
                    return;
                }
                if (flavor.equals(DataFlavor.javaFileListFlavor))
                {
                    java.util.List fileList = null;
                    fileList = (java.util.List)data;
                    if (fileList != null)
                    {
                        Iterator iterator = fileList.iterator();
                        while (iterator.hasNext())
                        {
                            openFile((File)iterator.next());
                        }
                    }
                }
                else
                {
                    String result = null;
                    if (flavor.equals(DataFlavor.stringFlavor))
                    {
                        result = (String)data;
                    }
                    else if (flavor.isMimeTypeEqual("text/plain"))
                    {
                        InputStream in = (InputStream)data;
                        StringBuffer resultBuffer = new StringBuffer();
                        boolean more = true;
                        String charset = flavor.getParameter("charset");
                        int ch;
                        if (charset.equals("unicode"))
                        {
                            boolean byteSequence = true;
                            int ch2 = -1;
                            do
                            {
                                try
                                {
                                    ch = in.read();
                                    ch2 = in.read();
                                }
                                catch (IOException ex)
                                {
                                    ex.printStackTrace();
                                    ch = -1;
                                }
                                if (ch != -1 && byteSequence)
                                {
                                    ch = (ch & 0xFF) | ((ch2 & 0xFF) << 8);
                                }
                                if (ch == 0xFFFE)
                                {
                                    byteSequence = false;
                                }
                                else if (ch != 0 && ch != -1)
                                {
                                    resultBuffer.append("" + (char)ch);
                                }
                                else
                                {
                                    more = false;
                                }
                            } while (more);
                        }
                        else
                        {
                            // hopefully ascii

                            do
                            {
                                try
                                {
                                    ch = in.read();
                                }
                                catch (IOException ex)
                                {
                                    ex.printStackTrace();
                                    ch = -1;
                                }
                                if (ch != 0 && ch != -1)
                                {
                                    resultBuffer.append("" + (char)ch);
                                }
                                else
                                {
                                    more = false;
                                }
                            } while (more);
                        }
                        result = resultBuffer.toString();
                    }
                    if (result != null)
                    {
                        Object target = ((DropTarget)event.getSource()).getComponent();
                        if ((target == jTabbedPane) || (target == contentPane))
                        {
                            if ((jCurrentTextArea.getText().length() > 0)
                                     || !(jTabbedPane.getTitleAt(jTabbedPane.getSelectedIndex()).equals(UNNAMED)))
                            {
                                addNewEditor();
                            }
                            jCurrentTextArea.setText(result);
                            currentModified = true;
                        }
                        else if (target == jCurrentTextArea)
                        {
                            Point pt = event.getLocation();
                            int pos = jCurrentTextArea.viewToModel(pt);
                            jCurrentTextArea.insert(result, pos);
                        }
                    }
                }
            }
            event.dropComplete(true);
        }
    }


    /**
     *  This class contains all options which are necessary to search in a text field.
     *
     * @author    ps
     */
    private class EditorFindOptions extends FindOptionsAbstract
    {

        private int lastOperation = 0;

        private final int FIND_OPERATION = 1;
        private final int REPLACE_OPERATION = 2;
        private final int REPLACEALL_OPERATION = 3;
        boolean firstSearch = true;
        private boolean allowNewStart = true;

        /**
         *  this method is used by findNext and replaceNext
         *
         * @return    Position of found string
         */
        private int internFindNext()
        {
            int pos;
            if (firstSearch && this.isSearchFromStart())
            {
                pos = 0;
            }
            else
            {
                if (isBackwards())
                {
                    pos = jCurrentTextArea.getSelectionStart() - 1;
                }
                else
                {
                    pos = jCurrentTextArea.getCaretPosition();
                }
            }
            firstSearch = false;
            int result = FindText.findNext(jCurrentTextArea.getText(), pos, this);
            if (result < 0)
            {
                if (this.isSearchAllFiles())
                {
                    int startSel = jTabbedPane.getSelectedIndex();
                    int sel = startSel;
                    do
                    {
                        if (this.isBackwards())
                        {
                            sel = sel - 1;
                            if (sel < 0)
                            {
                                sel = jTabbedPane.getTabCount() - 1;
                            }
                        }
                        else
                        {
                            sel = sel + 1;
                            if (sel == jTabbedPane.getTabCount())
                            {
                                sel = 0;
                            }
                        }
                        jTabbedPane.setSelectedIndex(sel);
                        result = FindText.findNext(jCurrentTextArea.getText(), 0, this);
                    } while ((result < 0) || (startSel == sel));
                }
                else if (allowNewStart)
                {
                    if (pos > 0)
                    {
                        result = FindText.findNext(jCurrentTextArea.getText(), 0, this);
                    }
                }
            }
            if (result >= 0)
            {
                jCurrentTextArea.select(result, result + this.getSearchText().length());
            }
            return (result);
        }


        /**
         * This method has to be called if a search action with the mode
         * search-from-beginning has to start from the beginning.<br>
         * There are some case where this does not has to be.
         */
        public void setNewText()
        {
            firstSearch = true;
        }


        /**
         *  Search for the next position of the search text
         *
         * @return    true if found
         */
        public boolean findNext()
        {
            lastOperation = FIND_OPERATION;
            return (internFindNext() >= 0);
        }


        /**
         *  Replaces the search text on the next found position
         *
         * @return    true if found
         */
        public boolean replaceNext()
        {
            int result = internFindNext();
            if (result >= 0)
            {
                jCurrentTextArea.replaceRange(this.getReplaceText(), result, result + this.getSearchText().length());
                if (!isBackwards())
                {
                    jCurrentTextArea.setCaretPosition(result + this.getReplaceText().length());
                }
                else
                {
                    jCurrentTextArea.setCaretPosition(result);
                }
            }
            lastOperation = REPLACE_OPERATION;
            return (result >= 0);
        }



        /**
         *  Replaces all found search texts
         *
         * @return    number of replaced texts
         */
        public int replaceAll()
        {
            int result = 0;
            allowNewStart = false;
            while (replaceNext())
            {
                result++;
            }
            allowNewStart = true;
            lastOperation = REPLACEALL_OPERATION;
            JOptionPane.showMessageDialog(null, "Es wurden " + result + " Ersetzungen vorgenommen!");
            return result;
        }


        /**
         *  Repeat the last operation on the current editor
         */
        public void doAgain()
        {
            boolean result = true;
            // not for replaceAll
            switch (lastOperation)
            {
                case FIND_OPERATION:
                    result = findNext();
                    break;
                case REPLACE_OPERATION:
                    result = replaceNext();
                    break;
                case REPLACEALL_OPERATION:
                    if (JOptionPane.showConfirmDialog(null, "Wollen Sie alle '" + getSearchText()
                             + "' durch '" + getReplaceText() + "' ersetzen?", "Alles Ersetzen", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)
                             == JOptionPane.YES_OPTION)
                    {
                        replaceAll();
                    }
                    break;
            }
            if (!result)
            {
                JOptionPane.showMessageDialog(null, "'" + getSearchText() + "' konnte nicht gefunden werden.");
            }
        }
    }


    /**
     *  A listener which registers changes in the data of an editor
     *
     * @author    ps
     */
    private class EditorDocumentListener implements DocumentListener
    {
        /**
         *  This method is called while every change in the editor data
         *
         * @param  doc  Document of the current editor
         */
        private void updateSizeInfo(Document doc)
        {
            updateSizeLabel(doc.getLength());
            ((EditorFindOptions)findOptions).setNewText();
            currentModified = true;
        }


        /**
         *  {@link javax.swing.event.DocumentListener#changedUpdate(DocumentEvent e)}
         */
        public void changedUpdate(DocumentEvent e)
        {
            updateSizeInfo(e.getDocument());
        }


        /**
         *  {@link javax.swing.event.DocumentListener#insertUpdate(DocumentEvent e)}
         */
        public void insertUpdate(DocumentEvent e)
        {
            updateSizeInfo(e.getDocument());
        }


        /**
         *  {@link javax.swing.event.DocumentListener#removeUpdate(DocumentEvent e)}
         */
        public void removeUpdate(DocumentEvent e)
        {
            updateSizeInfo(e.getDocument());
        }
    }

}
