package de.pschuett.find;

/**
 * Interface for search options
 *
 * @author 	ps
 */
public interface FindOptions
{
    /**
     * Returns the text for which is searched
     *
     * @return	search text
     */
    public String getSearchText();


    /**
     * Sets the text for which is searched
     *
     * @param	new search text
     */
    public void setSearchText(String aText);


    /**
     * Returns the text which shall replace the search text
     *
     * @return	replace text
     */
    public String getReplaceText();


    /**
     * Sets the text which shall replace the search text
     *
     * @param	new replace text
     */
    public void setReplaceText(String aText);


    /**
     * Returns true if backward search is active
     *
     * @return	true for backward search
     */
    public boolean isBackwards();


    /**
     * Activates or deactivates the backward search modus
     *
     * @param	true for backward search modus
     */
    public void setBackwards(boolean aBack);


    /**
     * Returns true if the whole word modus is active
     *
     * @return	True for whole word modus
     */
    public boolean isWholeWord();


    /**
     * Activates or deactivates the whole word modus
     *
     * @param	true for whole word modus
     */
    public void setWholeWord(boolean aValue);


    /**
     * Returns true if a search over all files is activated
     *
     * @return	true for all files mode
     */
    public boolean isSearchAllFiles();


    /**
     * Activates of deactivates the search mode over all files
     *
     * @param	True for search over all files
     */
    public void setSearchAllFiles(boolean aValue);


    /**
     * Search for the next position of the search text
     *
     * @return	true if found
     */
    public boolean findNext();


    /**
     * Replaces the search text on the next found position
     *
     * @return	true if found
     */
    public boolean replaceNext();


    /**
     * Replaces all found search texts
     *
     * @return	number of replaced texts
     */
    public int replaceAll();


    /**
     * Returns true if case sensitive search is activate
     *
     * @return	true for case sensitive search
     */
    public boolean isCaseSensitive();


    /**
     * Activates or deactivates case sensitive search
     *
     * @param	true for case sensitive search
     */
    public void setCaseSensitive(boolean aValue);


    /**
     * Returns if the search starts from the beginning of the text.<br>
     * Otherwise it starts from the cursor position.
     *
     * @return	true if search starts from the beginning of the text
     */
    public boolean isSearchFromStart();


    /**
     * Activates or deactivates that the search starts from the beginning of the text<br>
     * Otherwise it start from the cursor position.
     *
     * @param	true search starts from the beginning of the text
     */
    public void setSearchFromStart(boolean aValue);
}


