package de.pschuett.find;

/**
 * Abstract class for search options<br>
 * All getter and setter methods are implemented.
 *
 * @author 	ps
 */
public abstract class FindOptionsAbstract implements FindOptions
{
    /**
     * backward search modus
     */
    private boolean mBackwards;

    /**
     * searches all files
     */
    private boolean mSearchAllFiles;

    /**
     * whole word modus
     */
    private boolean mWholeWord;

    /**
     * searching from the beginning of the text
     */
    private boolean mSearchFromStart;

    /**
     * case sensitive modus
     */
    private boolean mCaseSensitive;

    /**
     * replace text
     */
    private String mReplaceText;

    /**
     * search text
     */
    private String mSearchText;

    /**
     * default constructor
     */
    public FindOptionsAbstract()
    {
    }


    /**
     * Returns the text for which is searched
     *
     * @return	search text
     */
    public String getSearchText()
    {
        return mSearchText;
    }


    /**
     * Sets the text for which is searched
     *
     * @param	new search text
     */
    public void setSearchText(String aText)
    {
        mSearchText = aText;
    }


    /**
     * Returns the text which shall replace the search text
     *
     * @return	replace text
     */
    public String getReplaceText()
    {
        return mReplaceText;
    }


    /**
     * Sets the text which shall replace the search text
     *
     * @param	new replace text
     */
    public void setReplaceText(String aText)
    {
        mReplaceText = aText;
    }


    /**
     * Returns true if backward search is active
     *
     * @return	true for backward search
     */
    public boolean isBackwards()
    {
        return mBackwards;
    }


    /**
     * Activates or deactivates the backward search modus
     *
     * @param	true for backward search modus
     */
    public void setBackwards(boolean aBack)
    {
        mBackwards = aBack;
    }


    /**
     * Returns true if the whole word modus is active
     *
     * @return	True for whole word modus
     */
    public boolean isWholeWord()
    {
        return mWholeWord;
    }


    /**
     * Activates or deactivates the whole word modus
     *
     * @param	true for whole word modus
     */
    public void setWholeWord(boolean aValue)
    {
        mWholeWord = aValue;
    }


    /**
     * Returns true if a search over all files is activated
     *
     * @return	true for all files mode
     */
    public boolean isSearchAllFiles()
    {
        return mSearchAllFiles;
    }


    /**
     * Activates of deactivates the search mode over all files
     *
     * @param	True for search over all files
     */
    public void setSearchAllFiles(boolean aValue)
    {
        mSearchAllFiles = aValue;
    }


    /**
     * Returns true if case sensitive search is activate
     *
     * @return	true for case sensitive search
     */
    public boolean isCaseSensitive()
    {
        return mCaseSensitive;
    }


    /**
     * Activates or deactivates case sensitive search
     *
     * @param	true for case sensitive search
     */
    public void setCaseSensitive(boolean aValue)
    {
        mCaseSensitive = aValue;
    }


    /**
     * Returns if the search starts from the beginning of the text.<br>
     * Otherwise it starts from the cursor position.
     *
     * @return	true if search starts from the beginning of the text
     */
    public boolean isSearchFromStart()
    {
        return mSearchFromStart;
    }


    /**
     * Activates or deactivates that the search starts from the beginning of the text<br>
     * Otherwise it start from the cursor position.
     *
     * @param	true search starts from the beginning of the text
     */
    public void setSearchFromStart(boolean aValue)
    {
        mSearchFromStart = aValue;
    }
}


